@@ -14,11 +14,20 @@ if is_service_enabled sl-api sl-eng; then
         install_senlin
         echo_summary "Installing senlinclient"
         install_senlinclient
+        if is_service_enabled horizon; then
+            echo_summary "Installing senlin dashboard"
+            install_senlin_dashboard
+        fi
         cleanup_senlin
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring senlin"
         configure_senlin
 
+        if is_service_enabled horizon; then
+            echo_summary "Configuring senlin dashboard"
+            config_senlin_dashboard
+        fi
+
         if is_service_enabled key; then
             create_senlin_accounts
         fi
@@ -38,6 +47,10 @@ if is_service_enabled sl-api sl-eng; then
 
     if [[ "$1" == "clean" ]]; then
         cleanup_senlin
+
+        if is_service_enabled horizon; then
+            cleanup_senlin_dashboard
+        fi
     fi
 fi
 
