@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from senlin.api.common import util
 from senlin.api.common import wsgi
 from senlin.common import context
 from senlin.common import exception
@@ -23,21 +24,22 @@ class TrustMiddleware(wsgi.Middleware):
     The extracted information is filled into the request context.
     Senlin engine will use this information for access control.
     """
-    def _get_trust(self, ctx):
+    def _get_trust(self, req):
         """List trusts with current user as the trustor.
 
-        :param ctx: The requesting context.
+        :param req: The WSGI request object.
         :return: ID of the trust or exception of InternalError.
         """
         rpcc = rpc.EngineClient()
 
-        cred_exists = False
-        res = rpcc.credential_get(ctx)
+        ctx = req.context
+        params = {'user': ctx.user, 'project': ctx.project}
+        obj = util.parse_request('CredentialGetRequest', req, params)
+        res = rpcc.call(ctx, 'credential_get', obj)
         if res:
             trust_id = res.get('trust', None)
             if trust_id:
                 return trust_id
-            cred_exists = True
 
         params = {
             'auth_url': ctx.auth_url,
@@ -46,7 +48,7 @@ class TrustMiddleware(wsgi.Middleware):
             'user_id': ctx.user,
         }
         kc = driver_base.SenlinDriver().identity(params)
-        service_cred = context.get_service_context()
+        service_cred = context.get_service_credentials()
         admin_id = kc.get_user_id(**service_cred)
         try:
             trust = kc.trust_get_by_trustor(ctx.user, admin_id, ctx.project)
@@ -54,19 +56,19 @@ class TrustMiddleware(wsgi.Middleware):
             if ex.code == 400:
                 trust = None
             else:
-                raise ex
+                raise
         if not trust:
             # Create a trust if no existing one found
             trust = kc.trust_create(ctx.user, admin_id, ctx.project, ctx.roles)
 
-        # update cache
-        if cred_exists:
-            rpcc.credential_update(ctx, trust.id)
-        else:
-            rpcc.credential_create(ctx, trust.id)
+        # If credential not exists, create it, otherwise update it.
+        cred = {'openstack': {'trust': trust.id}}
+        params = {'cred': cred}
+        obj = util.parse_request('CredentialCreateRequest', req, params)
+        rpcc.call(ctx, 'credential_create', obj)
 
         return trust.id
 
     def process_request(self, req):
-        trust_id = self._get_trust(req.context)
+        trust_id = self._get_trust(req)
         req.context.trusts = trust_id
