@@ -121,7 +121,7 @@ class ClusterController(wsgi.Controller):
 
     @wsgi.Controller.api_version('1.3')
     def _do_replace_nodes(self, req, cluster_id, data):
-        nodes = data.get('nodes', [])
+        nodes = data.get('nodes', {})
         if not nodes or not isinstance(nodes, dict):
             msg = _("The data provided is not a map")
             raise exc.HTTPBadRequest(msg)
@@ -133,23 +133,26 @@ class ClusterController(wsgi.Controller):
 
     def _do_resize(self, req, cluster_id, data):
         params = {}
-        if data.get(consts.ADJUSTMENT_TYPE, None):
-            params['adjustment_type'] = data.get(consts.ADJUSTMENT_TYPE)
-        if data.get(consts.ADJUSTMENT_NUMBER, None) is not None:
-            params['number'] = data.get(consts.ADJUSTMENT_NUMBER)
-        if data.get(consts.ADJUSTMENT_MIN_SIZE, None) is not None:
-            params['min_size'] = data.get(consts.ADJUSTMENT_MIN_SIZE)
-        if data.get(consts.ADJUSTMENT_MAX_SIZE, None) is not None:
-            params['max_size'] = data.get(consts.ADJUSTMENT_MAX_SIZE)
-        if data.get(consts.ADJUSTMENT_MIN_STEP, None) is not None:
-            params['min_step'] = data.get(consts.ADJUSTMENT_MIN_STEP)
-        if data.get(consts.ADJUSTMENT_STRICT, None) is not None:
-            params['strict'] = data.get(consts.ADJUSTMENT_STRICT)
+
+        for key in [consts.ADJUSTMENT_TYPE, consts.ADJUSTMENT_NUMBER,
+                    consts.ADJUSTMENT_MIN_SIZE, consts.ADJUSTMENT_MAX_SIZE]:
+            if data.get(key, None) is not None:
+                params[key] = data.get(key)
+
+        adj_type = data.get(consts.ADJUSTMENT_TYPE, None)
+        min_step = data.get(consts.ADJUSTMENT_MIN_STEP, None)
+        if ((adj_type == consts.CHANGE_IN_PERCENTAGE) and
+                min_step is not None):
+            params[consts.ADJUSTMENT_MIN_STEP] = min_step
 
         if not params:
             msg = _("Not enough parameters to do resize action.")
             raise exc.HTTPBadRequest(msg)
 
+        strict = data.get(consts.ADJUSTMENT_STRICT, None)
+        if strict is not None:
+            params[consts.ADJUSTMENT_STRICT] = strict
+
         params['identity'] = cluster_id
         obj = util.parse_request('ClusterResizeRequest', req, params)
 
@@ -184,15 +187,14 @@ class ClusterController(wsgi.Controller):
     def _do_scale_in(self, req, cid, data):
         count = data.get('count', None)
         params = {'identity': cid}
+
         if count is not None:
             params['count'] = count
+
         obj = util.parse_request('ClusterScaleInRequest', req, params)
         return self.rpc_client.call(req.context, 'cluster_scale_in', obj)
 
     def _do_policy_attach(self, req, cid, data):
-        if not isinstance(data, dict):
-            msg = _("The data provided is not a map")
-            raise exc.HTTPBadRequest(msg)
         params = {'identity': cid}
         params.update(data)
         obj = util.parse_request('ClusterAttachPolicyRequest', req, params)
@@ -200,9 +202,6 @@ class ClusterController(wsgi.Controller):
                                     'cluster_policy_attach', obj)
 
     def _do_policy_detach(self, req, cid, data):
-        if not isinstance(data, dict):
-            msg = _("The data provided is not a map")
-            raise exc.HTTPBadRequest(msg)
         params = {'identity': cid}
         params.update(data)
 
@@ -211,9 +210,6 @@ class ClusterController(wsgi.Controller):
                                     'cluster_policy_detach', obj)
 
     def _do_policy_update(self, req, cid, data):
-        if not isinstance(data, dict):
-            msg = _("The data provided is not a map")
-            raise exc.HTTPBadRequest(msg)
         params = {'identity': cid}
         params.update(data)
 
@@ -252,6 +248,10 @@ class ClusterController(wsgi.Controller):
 
         do_func = getattr(self, do_func_name)
         data = body.get(this_action, {})
+        if not isinstance(data, dict):
+            msg = _("The data provided is not a map")
+            raise exc.HTTPBadRequest(msg)
+
         res = do_func(req, cluster_id, data)
 
         location = {'location': '/actions/%s' % res['action']}
