@@ -70,6 +70,23 @@ class NodeController(wsgi.Controller):
         }
         return result
 
+    @wsgi.Controller.api_version('1.7')
+    @util.policy_enforce
+    def adopt(self, req, body):
+        """Adopt a node for management."""
+        obj = util.parse_request('NodeAdoptRequest', req, body)
+        node = self.rpc_client.call(req.context, 'node_adopt', obj)
+        return {'node': node}
+
+    @wsgi.Controller.api_version('1.7')
+    @util.policy_enforce
+    def adopt_preview(self, req, body):
+        """Preview a node adoption."""
+        # make sure we will fall into the preview path
+        obj = util.parse_request('NodeAdoptPreviewRequest', req, body)
+        node = self.rpc_client.call(req.context, 'node_adopt_preview', obj)
+        return {'node_profile': node}
+
     @util.policy_enforce
     def get(self, req, node_id):
         params = {'identity': node_id}
