@@ -24,7 +24,7 @@ class VersionController(object):
     # This includes any semantic changes which may not affect the input or
     # output formats or even originate in the API code layer.
     _MIN_API_VERSION = "1.0"
-    _MAX_API_VERSION = "1.5"
+    _MAX_API_VERSION = "1.7"
 
     DEFAULT_API_VERSION = _MIN_API_VERSION
 
@@ -56,7 +56,7 @@ class VersionController(object):
                 "href": "/v1",
                 "rel": "self"}, {
                 "rel": "help",
-                "href": "http://developer.openstack.org/api-ref/clustering"
+                "href": "https://developer.openstack.org/api-ref/clustering"
             }],
             "min_version": cls._MIN_API_VERSION,
             "max_version": cls._MAX_API_VERSION,
