@@ -10,63 +10,81 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-'''
-Utilities module.
-'''
+"""
+Common utilities module.
+"""
 
 import random
 import string
 
-from cryptography.fernet import Fernet
-import requests
-from requests import exceptions
-from six.moves import urllib
-
+from jsonpath_rw import parse
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 from oslo_utils import strutils
+from oslo_utils import timeutils
+import requests
+import six
+from six.moves import urllib
 
+from senlin.common import consts
 from senlin.common import exception
 from senlin.common.i18n import _
-from senlin.common.i18n import _LI
+from senlin.objects import service as service_obj
 
 cfg.CONF.import_opt('max_response_size', 'senlin.common.config')
+cfg.CONF.import_opt('periodic_interval', 'senlin.common.config')
+
 LOG = logging.getLogger(__name__)
+_ISO8601_TIME_FORMAT = '%Y-%m-%dT%H:%M:%S'
 
 
 class URLFetchError(exception.Error, IOError):
     pass
 
 
-def parse_int_param(name, value, allow_zero=True, allow_negative=False,
-                    lower_limit=None, upper_limit=None):
-    if value is None:
+def get_positive_int(v):
+    """Util function converting/checking a value of positive integer.
+
+    :param v: A value to be checked.
+    :returns: (b, v) where v is (converted) value if bool is True.
+              b is False if the value fails validation.
+    """
+    if strutils.is_int_like(v):
+        count = int(v)
+        if count > 0:
+            return True, count
+    return False, 0
+
+
+def parse_level_values(values):
+    """Parse a given list of level values to numbers.
+
+    :param values: A list of event level values.
+    :return: A list of translated values.
+    """
+    if not isinstance(values, list):
+        values = [values]
+    result = []
+    for v in values:
+        if v in consts.EVENT_LEVELS:
+            result.append(consts.EVENT_LEVELS[v])
+        elif isinstance(v, int):
+            result.append(v)
+
+    if result == []:
         return None
-
-    if value in ('0', 0):
-        if allow_zero:
-            return int(value)
-        raise exception.InvalidParameter(name=name, value=value)
-
-    try:
-        result = int(value)
-    except (TypeError, ValueError):
-        raise exception.InvalidParameter(name=name, value=value)
-    else:
-        if any([(allow_negative is False and result < 0),
-                (lower_limit and result < lower_limit),
-                (upper_limit and result > upper_limit)]):
-            raise exception.InvalidParameter(name=name, value=value)
-
     return result
 
 
-def parse_bool_param(name, value):
-    if str(value).lower() not in ('true', 'false'):
-        raise exception.InvalidParameter(name=name, value=str(value))
+def level_from_number(value):
+    """Parse a given level value(from number to string).
 
-    return strutils.bool_from_string(value, strict=True)
+    :param value: event level number.
+    :return: A translated value.
+    """
+    n = int(value)
+    levels = {value: key for key, value in consts.EVENT_LEVELS.items()}
+    return levels.get(n, None)
 
 
 def url_fetch(url, allowed_schemes=('http', 'https')):
@@ -77,7 +95,7 @@ def url_fetch(url, allowed_schemes=('http', 'https')):
     the allowed_schemes argument.
     Raise an IOError if getting the data fails.
     '''
-    LOG.info(_LI('Fetching data from %s'), url)
+    LOG.info('Fetching data from %s', url)
 
     components = urllib.parse.urlparse(url)
 
@@ -110,40 +128,68 @@ def url_fetch(url, allowed_schemes=('http', 'https')):
                                     " bytes)" % cfg.CONF.max_response_size)
         return result
 
-    except exceptions.RequestException as ex:
+    except requests.exceptions.RequestException as ex:
         raise URLFetchError(_('Failed to retrieve data: %s') % ex)
 
 
-def encrypt(msg):
-    '''Encrypt message with random key.
-
-    :param msg: message to be encrypted
-    :returns: encrypted msg and key to decrypt
-    '''
-    password = Fernet.generate_key()
-    f = Fernet(password)
-    key = f.encrypt(encodeutils.safe_encode(msg))
-    return encodeutils.safe_decode(password), encodeutils.safe_decode(key)
-
-
-def decrypt(msg, key):
-    '''Decrypt message using provided key.
-
-    :param msg: encrypted message
-    :param key: key used to decrypt
-    :returns: decrypted message string
-    '''
-    f = Fernet(encodeutils.safe_encode(msg))
-    msg = f.decrypt(encodeutils.safe_encode(key))
-
-    return encodeutils.safe_decode(msg)
-
-
 def random_name(length=8):
     if length <= 0:
         return ''
 
     lead = random.choice(string.ascii_letters)
     tail = ''.join(random.choice(string.ascii_letters + string.digits)
-                   for i in range(length-1))
+                   for i in range(length - 1))
     return lead + tail
+
+
+def isotime(at):
+    """Stringify time in ISO 8601 format.
+
+    oslo.versionedobject is using this function for datetime formatting.
+    """
+    if at is None:
+        return None
+
+    st = at.strftime(_ISO8601_TIME_FORMAT)
+    tz = at.tzinfo.tzname(None) if at.tzinfo else 'UTC'
+    st += ('Z' if tz == 'UTC' else tz)
+    return st
+
+
+def get_path_parser(path):
+    """Get a JsonPath parser based on a path string.
+
+    :param path: A string containing a JsonPath.
+    :returns: A parser used for path matching.
+    :raises: An exception of `BadRequest` if the path fails validation.
+    """
+    try:
+        expr = parse(path)
+    except Exception as ex:
+        error_text = six.text_type(ex)
+        error_msg = error_text.split(':', 1)[1]
+        raise exception.BadRequest(
+            msg=_("Invalid attribute path - %s") % error_msg.strip())
+
+    return expr
+
+
+def is_engine_dead(ctx, engine_id, duration=None):
+    """Check if an engine is dead.
+
+    If engine hasn't reported its status for the given duration, it is treated
+    as a dead engine.
+
+    :param ctx: A request context.
+    :param engine_id: The ID of the engine to test.
+    :param duration: The time duration in seconds.
+    """
+    if not duration:
+        duration = 2 * cfg.CONF.periodic_interval
+
+    eng = service_obj.Service.get(ctx, engine_id)
+    if not eng:
+        return True
+    if timeutils.is_older_than(eng.updated_at, duration):
+        return True
+    return False
