@@ -98,13 +98,14 @@ def node_get_all(context, cluster_id=None, limit=None, marker=None, sort=None,
                              project_safe=project_safe)
 
 
-def node_get_all_by_cluster(context, cluster_id, project_safe=True):
-    return IMPL.node_get_all_by_cluster(context, cluster_id,
+def node_get_all_by_cluster(context, cluster_id, filters=None,
+                            project_safe=True):
+    return IMPL.node_get_all_by_cluster(context, cluster_id, filters=filters,
                                         project_safe=project_safe)
 
 
-def node_ids_by_cluster(context, cluster_id):
-    return IMPL.node_ids_by_cluster(context, cluster_id)
+def node_ids_by_cluster(context, cluster_id, filters=None):
+    return IMPL.node_ids_by_cluster(context, cluster_id, filters=None)
 
 
 def node_count_by_cluster(context, cluster_id, **kwargs):
@@ -344,8 +345,11 @@ def action_check_status(context, action_id, timestamp):
     return IMPL.action_check_status(context, action_id, timestamp)
 
 
-def action_delete_by_target(context, target, exceptions=None):
-    return IMPL.action_check_status(context, target, exceptions=exceptions)
+def action_delete_by_target(context, target, action=None,
+                            action_excluded=None, status=None):
+    return IMPL.action_delete_by_target(context, target, action=action,
+                                        action_excluded=action_excluded,
+                                        status=status)
 
 
 def dependency_add(context, depended, dependent):
@@ -431,30 +435,33 @@ def receiver_delete(context, receiver_id):
     return IMPL.receiver_delete(context, receiver_id)
 
 
-def service_create(context, service_id, host=None, binary=None,
-                   topic=None):
-    return IMPL.service_create(context, service_id=service_id, host=host,
-                               binary=binary, topic=topic)
+def receiver_update(context, receiver_id, values):
+    return IMPL.receiver_update(context, receiver_id, values)
 
 
-def service_update(context, service_id, values=None):
-    return IMPL.service_update(context, service_id, values=values)
+def service_create(service_id, host=None, binary=None, topic=None):
+    return IMPL.service_create(service_id, host=host, binary=binary,
+                               topic=topic)
 
 
-def service_delete(context, service_id):
-    return IMPL.service_delete(context, service_id)
+def service_update(service_id, values=None):
+    return IMPL.service_update(service_id, values=values)
 
 
-def service_get(context, service_id):
-    return IMPL.service_get(context, service_id)
+def service_delete(service_id):
+    return IMPL.service_delete(service_id)
 
 
-def service_get_all(context):
-    return IMPL.service_get_all(context)
+def service_get(service_id):
+    return IMPL.service_get(service_id)
 
 
-def gc_by_engine(context, engine_id):
-    return IMPL.gc_by_engine(context, engine_id)
+def service_get_all():
+    return IMPL.service_get_all()
+
+
+def gc_by_engine(engine_id):
+    return IMPL.gc_by_engine(engine_id)
 
 
 def registry_create(context, cluster_id, check_type, interval, params,
@@ -475,6 +482,10 @@ def registry_claim(context, engine_id):
     return IMPL.registry_claim(context, engine_id)
 
 
+def registry_get(context, cluster_id):
+    return IMPL.registry_get(context, cluster_id)
+
+
 def db_sync(engine, version=None):
     """Migrate the database to `version` or the most recent version."""
     return IMPL.db_sync(engine, version=version)
