@@ -272,17 +272,25 @@ def node_get_all(context, cluster_id=None, limit=None, marker=None, sort=None,
                                    marker=marker, sort_dirs=dirs).all()
 
 
-def node_get_all_by_cluster(context, cluster_id, project_safe=True):
-    return _query_node_get_all(context, cluster_id=cluster_id,
-                               project_safe=project_safe).all()
+def node_get_all_by_cluster(context, cluster_id, filters=None,
+                            project_safe=True):
+
+    query = _query_node_get_all(context, cluster_id=cluster_id,
+                                project_safe=project_safe)
+    if filters:
+        query = utils.exact_filter(query, models.Node, filters)
+
+    return query.all()
 
 
-def node_ids_by_cluster(context, cluster_id):
+def node_ids_by_cluster(context, cluster_id, filters=None):
     """an internal API for getting node IDs."""
     with session_for_read() as session:
-        nodes = session.query(models.Node.id).filter_by(
-            cluster_id=cluster_id).all()
-        return [n[0] for n in nodes]
+        query = session.query(models.Node.id).filter_by(cluster_id=cluster_id)
+        if filters:
+            query = utils.exact_filter(query, models.Node, filters)
+
+        return [n[0] for n in query.all()]
 
 
 def node_count_by_cluster(context, cluster_id, **kwargs):
@@ -328,7 +336,7 @@ def node_add_dependents(context, depended, dependent, dep_type=None):
     :param dependent: ID of the dependent node or profile which has
                      dependencies on depended node.
     :param dep_type: The type of dependency. It can be 'node' indicating a
-                     dependency beween two nodes; or 'profile' indicating a
+                     dependency between two nodes; or 'profile' indicating a
                      dependency from profile to node.
     :raises ResourceNotFound: The specified node does not exist in database.
     """
@@ -354,7 +362,7 @@ def node_remove_dependents(context, depended, dependent, dep_type=None):
     :param dependent: ID of the node or profile which has dependencies on
                      the depended node.
     :param dep_type: The type of dependency. It can be 'node' indicating a
-                     dependency beween two nodes; or 'profile' indicating a
+                     dependency between two nodes; or 'profile' indicating a
                      dependency from profile to node.
 
     :raises ResourceNotFound: The specified node does not exist in database.
@@ -1038,15 +1046,6 @@ def action_check_status(context, action_id, timestamp):
         return action.status
 
 
-def action_delete_by_target(context, target, exceptions=None):
-    with session_for_write() as session:
-        q = session.query(models.Action).\
-            filter(models.Action.target == target)
-        if exceptions:
-            q = q.filter(~models.Action.action.in_(exceptions))
-        return q.delete(synchronize_session='fetch')
-
-
 def dependency_get_depended(context, action_id):
     with session_for_read() as session:
         q = session.query(models.ActionDependency).filter_by(
@@ -1098,6 +1097,8 @@ def dependency_add(context, depended, dependent):
                  synchronize_session='fetch')
 
 
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
 def action_mark_succeeded(context, action_id, timestamp):
     with session_for_write() as session:
 
@@ -1136,6 +1137,8 @@ def _mark_failed(session, action_id, timestamp, reason=None):
         _mark_failed(session, d, timestamp)
 
 
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
 def action_mark_failed(context, action_id, timestamp, reason=None):
     with session_for_write() as session:
         _mark_failed(session, action_id, timestamp, reason)
@@ -1161,6 +1164,8 @@ def _mark_cancelled(session, action_id, timestamp, reason=None):
         _mark_cancelled(session, d, timestamp)
 
 
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
 def action_mark_cancelled(context, action_id, timestamp, reason=None):
     with session_for_write() as session:
         _mark_cancelled(session, action_id, timestamp, reason)
@@ -1270,6 +1275,31 @@ def action_delete(context, action_id):
         session.delete(action)
 
 
+def action_delete_by_target(context, target, action=None,
+                            action_excluded=None, status=None,
+                            project_safe=True):
+    if action and action_excluded:
+        msg = _("action and action_excluded cannot be specified "
+                "both.")
+        LOG.warning(msg)
+        return None
+
+    with session_for_write() as session:
+        q = session.query(models.Action).\
+            filter_by(target=target)
+
+        if project_safe:
+            q = q.filter_by(project=context.project)
+
+        if action:
+            q = q.filter(models.Action.action.in_(action))
+        if action_excluded:
+            q = q.filter(~models.Action.action.in_(action_excluded))
+        if status:
+            q = q.filter(models.Action.status.in_(status))
+        return q.delete(synchronize_session='fetch')
+
+
 # Receivers
 def receiver_create(context, values):
     with session_for_write() as session:
@@ -1325,8 +1355,18 @@ def receiver_delete(context, receiver_id):
         session.delete(receiver)
 
 
-def service_create(context, service_id, host=None, binary=None,
-                   topic=None):
+def receiver_update(context, receiver_id, values):
+    with session_for_write() as session:
+        receiver = session.query(models.Receiver).get(receiver_id)
+        if not receiver:
+            raise exception.ResourceNotFound(type='receiver', id=receiver_id)
+
+        receiver.update(values)
+        receiver.save(session)
+        return receiver
+
+
+def service_create(service_id, host=None, binary=None, topic=None):
     with session_for_write() as session:
         time_now = timeutils.utcnow(True)
         svc = models.Service(id=service_id, host=host, binary=binary,
@@ -1336,7 +1376,7 @@ def service_create(context, service_id, host=None, binary=None,
         return svc
 
 
-def service_update(context, service_id, values=None):
+def service_update(service_id, values=None):
     with session_for_write() as session:
         service = session.query(models.Service).get(service_id)
         if not service:
@@ -1351,21 +1391,23 @@ def service_update(context, service_id, values=None):
         return service
 
 
-def service_delete(context, service_id):
+def service_delete(service_id):
     with session_for_write() as session:
         session.query(models.Service).filter_by(
             id=service_id).delete(synchronize_session='fetch')
 
 
-def service_get(context, service_id):
-    return model_query(context, models.Service).get(service_id)
+def service_get(service_id):
+    with session_for_read() as session:
+        return session.query(models.Service).get(service_id)
 
 
-def service_get_all(context):
-    return model_query(context, models.Service).all()
+def service_get_all():
+    with session_for_read() as session:
+        return session.query(models.Service).all()
 
 
-def gc_by_engine(context, engine_id):
+def gc_by_engine(engine_id):
     # Get all actions locked by an engine
     with session_for_write() as session:
         q_actions = session.query(models.Action).filter_by(owner=engine_id)
@@ -1381,7 +1423,7 @@ def gc_by_engine(context, engine_id):
                 if not res:
                     _release_cluster_lock(session, cl, a.id, 1)
 
-            # mark action failed and relase lock
+            # mark action failed and release lock
             _mark_failed(session, a.id, timestamp, reason="Engine failure")
 
 
@@ -1417,8 +1459,10 @@ def registry_claim(context, engine_id):
         if svc_ids:
             q_reg = q_reg.filter(
                 models.HealthRegistry.engine_id.notin_(svc_ids))
-        q_reg.update({'engine_id': engine_id}, synchronize_session=False)
+
         result = q_reg.all()
+        q_reg.update({'engine_id': engine_id}, synchronize_session=False)
+
         return result
 
 
@@ -1431,6 +1475,14 @@ def registry_delete(context, cluster_id):
         session.delete(registry)
 
 
+def registry_get(context, cluster_id):
+    with session_for_read() as session:
+        registry = session.query(models.HealthRegistry).filter_by(
+            cluster_id=cluster_id).first()
+
+        return registry
+
+
 # Utils
 def db_sync(engine, version=None):
     """Migrate the database to `version` or the most recent version."""
