@@ -13,12 +13,9 @@
 import copy
 
 from oslo_config import cfg
-from oslo_log import log as logging
 
 from senlin.engine import environment
 
-LOG = logging.getLogger(__name__)
-
 CONF = cfg.CONF
 
 
@@ -39,11 +36,12 @@ class SenlinDriver(object):
 
         backend = environment.global_env().get_driver(backend_name)
 
-        # TODO(Yanyan Hu): Use openstack compute driver(nova_v2)
-        # as the start point of using senlin generic driver.
         self.compute = backend.compute
         self.loadbalancing = backend.loadbalancing
         self.network = backend.network
         self.orchestration = backend.orchestration
         self.telemetry = backend.telemetry
         self.identity = backend.identity
+        self.message = backend.message
+        self.workflow = backend.workflow
+        self.block_storage = backend.block_storage
