@@ -11,13 +11,13 @@
 # under the License.
 
 from oslo_config import cfg
-from oslo_log import log as logging
+from oslo_log import log
 
 from senlin.drivers import base
 from senlin.drivers.openstack import sdk
 
+LOG = log.getLogger(__name__)
 CONF = cfg.CONF
-LOG = logging.getLogger(__name__)
 
 
 class KeystoneClient(base.DriverBase):
@@ -28,51 +28,6 @@ class KeystoneClient(base.DriverBase):
         self.conn = sdk.create_connection(params)
         self.session = self.conn.session
 
-    @sdk.translate_exception
-    def trust_list(self, **query):
-        trusts = [t for t in self.conn.identity.trusts(**query)]
-        return trusts
-
-    @sdk.translate_exception
-    def trust_delete(self, trust_id):
-        self.conn.identity.delete_trust(trust_id, ignore_missing=True)
-        return
-
-    @sdk.translate_exception
-    def user_get(self, name_or_id, ignore_missing=False):
-        # Note: Since default keystone policy will block non-admin user to
-        # get/list user(s), this function may not be workable. Pleaes use
-        # get_user_id method to get the ID of user if needed.
-        user = self.conn.identity.find_user(
-            name_or_id, ignore_missing=ignore_missing)
-        return user
-
-    @sdk.translate_exception
-    def endpoint_get(self, service_id, region=None, interface=None):
-        '''Utility function to get endpoints of a service.'''
-        filters = {
-            'service_id': service_id,
-        }
-        if region:
-            filters['region'] = region
-        if interface:
-            filters['interface'] = interface
-
-        endpoints = [e for e in self.conn.identity.endpoints(**filters)]
-        return endpoints[0] if endpoints else None
-
-    @sdk.translate_exception
-    def service_get(self, service_type, name=None):
-        '''Utility function to get service detail based on name and type.'''
-        filters = {
-            'type': service_type,
-        }
-        if name:
-            filters['name'] = name
-
-        services = [s for s in self.conn.identity.services(**filters)]
-        return services[0] if services else None
-
     @sdk.translate_exception
     def trust_get_by_trustor(self, trustor, trustee=None, project=None):
         '''Get trust by trustor.
@@ -108,7 +63,7 @@ class KeystoneClient(base.DriverBase):
 
         :param trustor: ID of the user who is the trustor.
         :param trustee: ID of the user who is the trustee.
-        :param project: Scope of of the trust which is a project ID.
+        :param project: Scope of the trust which is a project ID.
         :param roles: List of roles the trustee will inherit from the trustor.
         :param impersonation: Whether the trustee is allowed to impersonate
                               the trustor.
@@ -137,17 +92,15 @@ class KeystoneClient(base.DriverBase):
         '''Get token using given credential'''
 
         access_info = sdk.authenticate(**creds)
-        token = access_info.auth_token
-        return token
+        return access_info['token']
 
     @classmethod
     @sdk.translate_exception
     def get_user_id(cls, **creds):
-        '''Get ID of the user with given creddential'''
+        '''Get ID of the user with given credential'''
 
         access_info = sdk.authenticate(**creds)
-        user_id = access_info.user_id
-        return user_id
+        return access_info['user_id']
 
     @classmethod
     def get_service_credentials(cls, **kwargs):
@@ -168,3 +121,34 @@ class KeystoneClient(base.DriverBase):
         }
         creds.update(**kwargs)
         return creds
+
+    @sdk.translate_exception
+    def validate_regions(self, regions):
+        """Check whether the given regions are valid.
+
+        :param regions: A list of regions for validation.
+        :returns: A list of regions that are found available on keystone.
+        """
+        region_list = self.conn.identity.regions()
+        known = [r['id'] for r in region_list]
+
+        validated = []
+        for r in regions:
+            if r in known:
+                validated.append(r)
+            else:
+                LOG.warning('Region %s is not found.', r)
+
+        return validated
+
+    @sdk.translate_exception
+    def get_senlin_endpoint(self):
+        '''Get Senlin service endpoint.'''
+        region = cfg.CONF.default_region_name
+        # TODO(Yanyan Hu): Currently, region filtering is unsupported in
+        # session.get_endpoint(). Need to propose fix to openstacksdk.
+        base = self.conn.session.get_endpoint(service_type='clustering',
+                                              interface='public',
+                                              region=region)
+
+        return base
