@@ -13,7 +13,6 @@
 from oslo_config import cfg
 from oslo_log import log
 
-from senlin.common.i18n import _LW
 from senlin.drivers import base
 from senlin.drivers.openstack import sdk
 
@@ -36,6 +35,14 @@ class NovaClient(base.DriverBase):
     def image_find(self, name_or_id, ignore_missing=False):
         return self.conn.compute.find_image(name_or_id, ignore_missing)
 
+    @sdk.translate_exception
+    def keypair_create(self, **attrs):
+        return self.conn.compute.create_keypair(**attrs)
+
+    @sdk.translate_exception
+    def keypair_delete(self, name_or_id, ignore_missing=False):
+        return self.conn.compute.delete_keypair(name_or_id, ignore_missing)
+
     @sdk.translate_exception
     def keypair_find(self, name_or_id, ignore_missing=False):
         return self.conn.compute.find_keypair(name_or_id, ignore_missing)
@@ -211,15 +218,14 @@ class NovaClient(base.DriverBase):
         :returns: A list of zones that are found available on Nova.
         """
         known = self.availability_zone_list()
-        names = [az['zoneName'] for az in known
-                 if az['zoneState']['available']]
+        names = [az.name for az in known if az.state['available']]
 
         found = []
         for az in azs:
             if az in names:
                 found.append(az)
             else:
-                LOG.warning(_LW("Availability zone '%s' is not available."),
+                LOG.warning("Availability zone '%s' is not available.",
                             az)
         return found
 
@@ -253,3 +259,22 @@ class NovaClient(base.DriverBase):
     def service_force_down(self, service):
         return self.conn.compute.force_service_down(service, service.host,
                                                     service.binary)
+
+    @sdk.translate_exception
+    def create_volume_attachment(self, server, **attr):
+        return self.conn.compute.create_volume_attachment(server, **attr)
+
+    @sdk.translate_exception
+    def delete_volume_attachment(self, volume_id, server, ignore_missing=True):
+        return self.conn.compute.delete_volume_attachment(
+            volume_id, server, ignore_missing=ignore_missing
+        )
+
+    @sdk.translate_exception
+    def server_floatingip_associate(self, server, address):
+        return self.conn.compute.add_floating_ip_to_server(server, address)
+
+    @sdk.translate_exception
+    def server_floatingip_disassociate(self, server, address):
+        return self.conn.compute.remove_floating_ip_from_server(server,
+                                                                address)
