@@ -20,8 +20,9 @@ from oslo_utils import timeutils
 from senlin.common import consts
 from senlin.common import context as req_context
 from senlin.common import exception
-from senlin.common.i18n import _, _LE
+from senlin.common.i18n import _
 from senlin.common import utils
+from senlin.engine import dispatcher
 from senlin.engine import event as EVENT
 from senlin.objects import action as ao
 from senlin.objects import cluster_policy as cpo
@@ -272,8 +273,8 @@ class Action(object):
             expected = (self.SUSPENDED)
 
         if self.status not in expected:
-            LOG.error(_LE("Action (%(id)s) is in status (%(actual)s) while "
-                          "expected status should be one of (%(expected)s)."),
+            LOG.error("Action (%(id)s) is in status (%(actual)s) while "
+                      "expected status must be one of (%(expected)s).",
                       dict(id=self.id[:8], expected=expected,
                            actual=self.status))
             return
@@ -318,6 +319,7 @@ class Action(object):
             # Action failed at the moment, but can be retried
             # We abandon it and then notify other dispatchers to execute it
             ao.Action.abandon(self.context, self.id)
+            dispatcher.start_action()
 
         if status == self.SUCCEEDED:
             EVENT.info(self, consts.PHASE_END, reason or 'SUCCEEDED')
@@ -336,8 +338,8 @@ class Action(object):
         return status
 
     def is_timeout(self):
-        time_lapse = wallclock() - self.start_time
-        return time_lapse > self.timeout
+        time_elapse = wallclock() - self.start_time
+        return time_elapse > self.timeout
 
     def _check_signal(self):
         # Check timeout first, if true, return timeout message
@@ -415,7 +417,7 @@ class Action(object):
                     self.data['status'] = policy_mod.CHECK_ERROR
                     self.data['reason'] = _('Policy %s cooldown is still '
                                             'in progress.') % policy.id
-                return
+                    return
 
             if method is not None:
                 method(cluster_id, self)
@@ -464,7 +466,7 @@ def ActionProc(context, action_id):
     # Step 1: materialize the action object
     action = Action.load(context, action_id=action_id, project_safe=False)
     if action is None:
-        LOG.error(_LE('Action "%s" could not be found.'), action_id)
+        LOG.error('Action "%s" could not be found.', action_id)
         return False
 
     EVENT.info(action, consts.PHASE_START)
@@ -481,10 +483,10 @@ def ActionProc(context, action_id):
         # executed.
         result = action.RES_ERROR
         reason = six.text_type(ex)
-        LOG.exception(_LE('Unexpected exception occurred during action '
-                          '%(action)s (%(id)s) execution: %(reason)s'),
+        LOG.exception('Unexpected exception occurred during action '
+                      '%(action)s (%(id)s) execution: %(reason)s',
                       {'action': action.action, 'id': action.id,
-                      'reason': reason})
+                       'reason': reason})
         success = False
     finally:
         # NOTE: locks on action is eventually released here by status update
