@@ -18,7 +18,7 @@ from oslo_utils import timeutils
 from osprofiler import profiler
 
 from senlin.common import consts
-from senlin.common.i18n import _, _LI
+from senlin.common.i18n import _
 from senlin.common import scaleutils
 from senlin.common import utils
 from senlin.engine.actions import base
@@ -184,9 +184,8 @@ class ClusterAction(base.Action):
 
     def _update_nodes(self, profile_id, nodes_obj):
         # Get batching policy data if any
-        fmt = _LI("Updating cluster '%(cluster)s': profile='%(profile)s'.")
+        fmt = _("Updating cluster '%(cluster)s': profile='%(profile)s'.")
         LOG.info(fmt, {'cluster': self.entity.id, 'profile': profile_id})
-        pause_time = 0
         plan = []
 
         pd = self.data.get('update', None)
@@ -254,11 +253,15 @@ class ClusterAction(base.Action):
             self.entity.set_status(self.context, consts.CS_ERROR, reason)
             return self.RES_ERROR, reason
 
+        config = self.inputs.get('config')
         name = self.inputs.get('name')
         metadata = self.inputs.get('metadata')
         timeout = self.inputs.get('timeout')
         profile_id = self.inputs.get('new_profile_id')
+        profile_only = self.inputs.get('profile_only')
 
+        if config is not None:
+            self.entity.config = config
         if name is not None:
             self.entity.name = name
         if metadata is not None:
@@ -273,6 +276,14 @@ class ClusterAction(base.Action):
                                     updated_at=timeutils.utcnow(True))
             return self.RES_OK, reason
 
+        # profile_only's type is bool
+        if profile_only:
+            self.entity.profile_id = profile_id
+            self.entity.eval_status(self.context, consts.CLUSTER_UPDATE,
+                                    profile_id=profile_id,
+                                    updated_at=timeutils.utcnow(True))
+            return self.RES_OK, reason
+
         # Update nodes with new profile
         result, reason = self._update_nodes(profile_id, self.entity.nodes)
         return result, reason
@@ -322,8 +333,6 @@ class ClusterAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        pause = 0
-        batch = 0
         reason = _('Deletion in progress.')
         self.entity.set_status(self.context, consts.CS_DELETING, reason)
         node_ids = [node.id for node in self.entity.nodes]
@@ -335,31 +344,28 @@ class ClusterAction(base.Action):
             }
         }
         self.data.update(data)
-        bp = self.data.get('delete', None)
-        # use policy data if any, or we specify batch as the length of
-        # nodes' list which means we treat is as one batch.
-        if bp is not None:
-            pause = bp.get('pause_time', 0)
-            batch = bp.get('batch_size', 0)
-        else:
-            batch = len(node_ids)
 
-        if batch != 0:
-            # sleep if needed
-            self._sleep(pause)
-            for start in range(0, len(node_ids), batch):
-                end = start + batch
-                result, reason = self._delete_nodes(node_ids[start:end])
-                if result != self.RES_OK:
-                    self.entity.eval_status(self.context,
-                                            consts.CLUSTER_DELETE)
-                    return result, reason
+        result, reason = self._delete_nodes(node_ids)
+        if result != self.RES_OK:
+            self.entity.eval_status(self.context, consts.CLUSTER_DELETE)
+            return result, reason
 
         res = self.entity.do_delete(self.context)
         if not res:
             self.entity.eval_status(self.context, consts.CLUSTER_DELETE)
             return self.RES_ERROR, _('Cannot delete cluster object.')
 
+        # Remove all action records which target on deleted
+        # cluster except the on-going CLUSTER_DELETE action from DB
+        try:
+            ao.Action.delete_by_target(
+                self.context, self.target,
+                action_excluded=[consts.CLUSTER_DELETE],
+                status=[consts.ACTION_SUCCEEDED,
+                        consts.ACTION_FAILED])
+        except Exception as ex:
+            LOG.warning('Failed to clean cluster action records: %s',
+                        ex)
         return self.RES_OK, reason
 
     @profiler.trace('ClusterAction.do_add_nodes', hide_args=False)
@@ -635,6 +641,22 @@ class ClusterAction(base.Action):
         self.entity.eval_status(self.context, consts.CLUSTER_CHECK)
         return res, reason
 
+    def _check_capacity(self):
+        cluster = self.entity
+
+        current = len(cluster.nodes)
+        desired = cluster.desired_capacity
+
+        if current < desired:
+            count = desired - current
+            self._create_nodes(count)
+
+        if current > desired:
+            count = current - desired
+            nodes = no.Node.get_all_by_cluster(self.context, cluster.id)
+            candidates = scaleutils.nodes_by_random(nodes, count)
+            self._delete_nodes(candidates)
+
     @profiler.trace('ClusterAction.do_recover', hide_args=False)
     def do_recover(self):
         """Handler for the CLUSTER_RECOVER action.
@@ -647,20 +669,28 @@ class ClusterAction(base.Action):
         pd = self.data.get('health', None)
         inputs = {}
         if pd:
+            check = self.data.get('check', False)
             recover_action = pd.get('recover_action', None)
             fencing = pd.get('fencing', None)
             if recover_action is not None:
-                # TODO(Qiming): Implement the support to action sequences
-                inputs['operation'] = recover_action[0]['name']
-                inputs['params'] = recover_action[0]['params'] or {}
+                inputs['operation'] = recover_action
             if fencing is not None and 'COMPUTE' in fencing:
-                inputs['params']['force'] = True
+                inputs['params'] = {'fence_compute': True}
+        else:
+            check = self.inputs.get('check', False)
+            recover_action = self.inputs.get('operation', None)
+            if recover_action is not None:
+                inputs['operation'] = recover_action
 
         children = []
         for node in self.entity.nodes:
-            if node.status == 'ACTIVE':
-                continue
             node_id = node.id
+            if check:
+                node = node_mod.Node.load(self.context, node_id=node_id)
+                node.do_check(self.context)
+
+            if node.status == consts.NS_ACTIVE:
+                continue
             action_id = base.Action.create(
                 self.context, node_id, consts.NODE_RECOVER,
                 name='node_recover_%s' % node_id[:8],
@@ -682,6 +712,10 @@ class ClusterAction(base.Action):
             if res != self.RES_OK:
                 reason = new_reason
 
+        check_capacity = self.inputs.get('check_capacity', False)
+        if check_capacity is True:
+            self._check_capacity()
+
         self.entity.eval_status(self.context, consts.CLUSTER_RECOVER)
         return res, reason
 
