@@ -12,6 +12,7 @@
 
 import eventlet
 
+from oslo_log import log as logging
 from osprofiler import profiler
 
 from senlin.common import consts
@@ -22,9 +23,12 @@ from senlin.engine import cluster as cm
 from senlin.engine import event as EVENT
 from senlin.engine import node as node_mod
 from senlin.engine import senlin_lock
+from senlin.objects import action as ao
 from senlin.objects import node as no
 from senlin.policies import base as pb
 
+LOG = logging.getLogger(__name__)
+
 
 class NodeAction(base.Action):
     """An action that can be performed on a cluster member (node)."""
@@ -115,11 +119,21 @@ class NodeAction(base.Action):
                 params = {'desired_capacity': desired}
             cluster.eval_status(self.context, consts.NODE_DELETE, **params)
 
-        if res:
-            return self.RES_OK, _('Node deleted successfully.')
-        else:
+        if not res:
             return self.RES_ERROR, _('Node deletion failed.')
 
+        # Remove all action records which target on deleted
+        # node except the on-going NODE_DELETE action from DB
+        try:
+            ao.Action.delete_by_target(
+                self.context, self.target,
+                action_excluded=[consts.NODE_DELETE],
+                status=[consts.ACTION_SUCCEEDED, consts.ACTION_FAILED])
+        except Exception as ex:
+            LOG.warning('Failed to clean node action records: %s',
+                        ex)
+        return self.RES_OK, _('Node deleted successfully.')
+
     @profiler.trace('NodeAction.do_update', hide_args=False)
     def do_update(self):
         """Handler for the NODE_UPDATE action.
@@ -176,9 +190,9 @@ class NodeAction(base.Action):
         """
         res = self.entity.do_check(self.context)
         if res:
-            return self.RES_OK, _('Node status is ACTIVE.')
+            return self.RES_OK, _('Node check succeeded.')
         else:
-            return self.RES_ERROR, _('Node status is not ACTIVE.')
+            return self.RES_ERROR, _('Node check failed.')
 
     @profiler.trace('NodeAction.do_recover', hide_args=False)
     def do_recover(self):
@@ -186,7 +200,7 @@ class NodeAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        res = self.entity.do_recover(self.context, **self.inputs)
+        res = self.entity.do_recover(self.context, self)
         if res:
             return self.RES_OK, _('Node recovered successfully.')
         else:
