@@ -16,12 +16,13 @@ from oslo_utils import timeutils
 
 from senlin.common import consts
 from senlin.common import exception
-from senlin.common.i18n import _, _LE
+from senlin.common.i18n import _
 from senlin.engine import cluster_policy as cpm
 from senlin.engine import health_manager
 from senlin.engine import node as node_mod
 from senlin.objects import cluster as co
 from senlin.objects import cluster_policy as cpo
+from senlin.objects import node as no
 from senlin.policies import base as pcb
 from senlin.profiles import base as pfb
 
@@ -72,6 +73,7 @@ class Cluster(object):
         self.data = kwargs.get('data', {})
         self.metadata = kwargs.get('metadata') or {}
         self.dependents = kwargs.get('dependents') or {}
+        self.config = kwargs.get('config') or {}
 
         # rt is a dict for runtime data
         self.rt = {
@@ -91,17 +93,14 @@ class Cluster(object):
         bindings = cpo.ClusterPolicy.get_all(context, self.id)
         for b in bindings:
             # Detect policy type conflicts
-            policy = pcb.Policy.load(context,
-                                     b.policy_id,
-                                     project_safe=False)
+            policy = pcb.Policy.load(context, b.policy_id, project_safe=False)
             policies.append(policy)
 
-        nodes = node_mod.Node.load_all(context, cluster_id=self.id)
         self.rt = {
             'profile': pfb.Profile.load(context,
                                         profile_id=self.profile_id,
                                         project_safe=False),
-            'nodes': [n for n in nodes],
+            'nodes': no.Node.get_all_by_cluster(context, self.id),
             'policies': policies
         }
 
@@ -130,6 +129,7 @@ class Cluster(object):
             'meta_data': self.metadata,
             'data': self.data,
             'dependents': self.dependents,
+            'config': self.config,
         }
 
         timestamp = timeutils.utcnow(True)
@@ -169,6 +169,7 @@ class Cluster(object):
             'data': obj.data,
             'metadata': obj.metadata,
             'dependents': obj.dependents,
+            'config': obj.config,
         }
 
         return cls(obj.name, obj.desired_capacity, obj.profile_id,
@@ -243,7 +244,7 @@ class Cluster(object):
         Set cluster status to CREATING.
         '''
         if self.status != consts.CS_INIT:
-            LOG.error(_LE('Cluster is in status "%s"'), self.status)
+            LOG.error('Cluster is in status "%s"', self.status)
             return False
 
         self.set_status(context, consts.CS_CREATING, _('Creation in progress'))
@@ -423,6 +424,13 @@ class Cluster(object):
             if node.id == node_id:
                 self.rt['nodes'].remove(node)
 
+    def update_node(self, nodes):
+        """Update cluster runtime data
+
+        :param nodes: List of node objects
+        """
+        self.rt['nodes'] = nodes
+
     @property
     def policies(self):
         return self.rt['policies']
@@ -496,6 +504,21 @@ class Cluster(object):
                     result.append(node)
         return result
 
+    def health_check(self, ctx):
+        """Check physical resources status
+
+        :param ctx: The context to operate node object
+        """
+        # Note this procedure is a pure sequential operation,
+        # its not suitable for large scale clusters.
+
+        old_nodes = self.nodes
+        for node in old_nodes:
+            node.do_check(ctx)
+
+        nodes = node_mod.Node.load_all(ctx, cluster_id=self.id)
+        self.update_node([n for n in nodes])
+
     def eval_status(self, ctx, operation, **params):
         """Re-evaluate cluster's health status.
 
