@@ -28,11 +28,8 @@ import time
 
 from senlin.common import consts
 from senlin.common import context
-from senlin.common.i18n import _LI, _LW
 from senlin.common import messaging as rpc
 from senlin import objects
-from senlin.objects.requests import actions as vora
-from senlin.objects.requests import clusters as vorc
 from senlin.rpc import client as rpc_client
 
 LOG = logging.getLogger(__name__)
@@ -50,7 +47,7 @@ def _chase_up(start_time, interval):
     # check if we have missed any intervals?
     missed = int((elapsed - 0.0000001) / interval)
     if missed >= 1:
-        LOG.warning(_LW("Poller missed % intervals for checking"), missed)
+        LOG.warning("Poller missed %s intervals for checking", missed)
     return (missed + 1) * interval - elapsed
 
 
@@ -88,10 +85,9 @@ class NovaNotificationEndpoint(object):
             }
             node_id = meta.get('cluster_node_id')
             if node_id:
-                LOG.info(_LI("Requesting node recovery: %s"), node_id)
-                ctx_dict = context.get_service_credentials(
-                    project=self.project_id, user=payload['user_id'])
-                ctx = context.RequestContext.from_dict(ctx_dict)
+                LOG.info("Requesting node recovery: %s", node_id)
+                ctx = context.get_service_context(project=self.project_id,
+                                                  user=payload['user_id'])
                 req = objects.NodeRecoverRequest(identity=node_id,
                                                  params=params)
                 self.rpc.call(ctx, 'node_recover', req)
@@ -99,14 +95,66 @@ class NovaNotificationEndpoint(object):
     def warn(self, ctxt, publisher_id, event_type, payload, metadata):
         meta = payload.get('metadata', {})
         if meta.get('cluster_id') == self.cluster_id:
-            LOG.warning("publisher=%s" % publisher_id)
-            LOG.warning("event_type=%s" % event_type)
+            LOG.warning("publisher=%s", publisher_id)
+            LOG.warning("event_type=%s", event_type)
 
     def debug(self, ctxt, publisher_id, event_type, payload, metadata):
         meta = payload.get('metadata', {})
         if meta.get('cluster_id') == self.cluster_id:
-            LOG.debug("publisher=%s" % publisher_id)
-            LOG.debug("event_type=%s" % event_type)
+            LOG.debug("publisher=%s", publisher_id)
+            LOG.debug("event_type=%s", event_type)
+
+
+class HeatNotificationEndpoint(object):
+
+    STACK_FAILURE_EVENTS = {
+        'orchestration.stack.delete.end': 'DELETE',
+    }
+
+    def __init__(self, project_id, cluster_id):
+        self.filter_rule = messaging.NotificationFilter(
+            publisher_id='^orchestration.*',
+            event_type='^orchestration\.stack\..*',
+            context={'project_id': '^%s$' % project_id})
+        self.project_id = project_id
+        self.cluster_id = cluster_id
+        self.rpc = rpc_client.EngineClient()
+
+    def info(self, ctxt, publisher_id, event_type, payload, metadata):
+        if event_type not in self.STACK_FAILURE_EVENTS:
+            return
+
+        tags = payload['tags']
+        if tags is None or tags == []:
+            return
+
+        cluster_id = None
+        node_id = None
+        for tag in tags:
+            if cluster_id is None:
+                start = tag.find('cluster_id')
+                if start == 0 and tag[11:] == self.cluster_id:
+                    cluster_id = tag[11:]
+            if node_id is None:
+                start = tag.find('cluster_node_id')
+                if start == 0:
+                    node_id = tag[16:]
+
+        if cluster_id is None or node_id is None:
+            return
+
+        params = {
+            'event': self.STACK_FAILURE_EVENTS[event_type],
+            'state': payload.get('state', 'Unknown'),
+            'stack_id': payload.get('stack_identity', 'Unknown'),
+            'timestamp': metadata['timestamp'],
+            'publisher': publisher_id,
+        }
+        LOG.info("Requesting stack recovery: %s", node_id)
+        ctx = context.get_service_context(project=self.project_id,
+                                          user=payload['user_identity'])
+        req = objects.NodeRecoverRequest(identity=node_id, params=params)
+        self.rpc.call(ctx, 'node_recover', req)
 
 
 def ListenerProc(exchange, project_id, cluster_id):
@@ -127,8 +175,12 @@ def ListenerProc(exchange, project_id, cluster_id):
             NovaNotificationEndpoint(project_id, cluster_id),
         ]
     else:  # heat notification
-        LOG.warning(_LW("Heat listener to be added."))
-        return
+        targets = [
+            messaging.Target(topic='notifications', exchange=exchange),
+        ]
+        endpoints = [
+            HeatNotificationEndpoint(project_id, cluster_id),
+        ]
 
     listener = messaging.get_notification_listener(
         transport, targets, endpoints, executor='threading',
@@ -158,12 +210,12 @@ class HealthManager(service.Service):
         The task is here so that the service always has something to wait()
         on, or else the process will exit.
         """
-        pass
+        self._load_runtime_registry()
 
     def _wait_for_action(self, ctx, action_id, timeout):
         done = False
         total_sleep = 0
-        req = vora.ActionGetRequest(identity=action_id)
+        req = objects.ActionGetRequest(identity=action_id)
         while total_sleep < timeout:
             action = self.rpc_client.call(ctx, 'action_get', req)
             if action['status'] in ['SUCCEEDED', 'FAILED', 'CANCELLED']:
@@ -190,32 +242,31 @@ class HealthManager(service.Service):
         start_time = timeutils.utcnow(True)
         cluster = objects.Cluster.get(self.ctx, cluster_id, project_safe=False)
         if not cluster:
-            LOG.warning(_LW("Cluster (%s) is not found."), cluster_id)
+            LOG.warning("Cluster (%s) is not found.", cluster_id)
             return _chase_up(start_time, timeout)
 
-        ctx_dict = context.get_service_credentials(user=cluster.user,
-                                                   project=cluster.project)
-        ctx = context.RequestContext.from_dict(ctx_dict)
+        ctx = context.get_service_context(user=cluster.user,
+                                          project=cluster.project)
         try:
-            req = vorc.ClusterCheckRequest(identity=cluster_id)
+            req = objects.ClusterCheckRequest(identity=cluster_id)
             action = self.rpc_client.call(ctx, 'cluster_check', req)
         except Exception as ex:
-            LOG.warning(_LW("Failed in triggering 'cluster_check' RPC for "
-                        "'%(c)s': %(r)s"),
+            LOG.warning("Failed in triggering 'cluster_check' RPC for "
+                        "'%(c)s': %(r)s",
                         {'c': cluster_id, 'r': six.text_type(ex)})
             return _chase_up(start_time, timeout)
 
         # wait for action to complete
         res, reason = self._wait_for_action(ctx, action['action'], timeout)
         if not res:
-            LOG.warning(_LW("%s"), reason)
+            LOG.warning("%s", reason)
             return _chase_up(start_time, timeout)
 
         # loop through nodes to trigger recovery
-        nodes = objects.Node.get_all(ctx, cluster_id=cluster_id)
+        nodes = objects.Node.get_all_by_cluster(ctx, cluster_id)
         for node in nodes:
             if node.status != 'ACTIVE':
-                LOG.info(_LI("Requesting node recovery: %s"), node.id)
+                LOG.info("Requesting node recovery: %s", node.id)
                 req = objects.NodeRecoverRequest(identity=node.id)
                 self.rpc_client.call(ctx, 'node_recover', req)
 
@@ -229,7 +280,7 @@ class HealthManager(service.Service):
         """
         cluster = objects.Cluster.get(self.ctx, cluster_id, project_safe=False)
         if not cluster:
-            LOG.warning(_LW("Cluster (%s) is not found."), cluster_id)
+            LOG.warning("Cluster (%s) is not found.", cluster_id)
             return
         profile = objects.Profile.get(self.ctx, cluster.profile_id,
                                       project_safe=False)
@@ -253,22 +304,22 @@ class HealthManager(service.Service):
         cid = entry['cluster_id']
         ctype = entry['check_type']
         if ctype == consts.NODE_STATUS_POLLING:
-            interval = min(entry['interval'], cfg.CONF.periodic_interval_max)
+            interval = min(entry['interval'], cfg.CONF.check_interval_max)
             timer = self.TG.add_dynamic_timer(self._poll_cluster,
                                               None,  # initial_delay
-                                              None,  # periodic_interval_max
+                                              None,  # check_interval_max
                                               cid, interval)
             entry['timer'] = timer
         elif ctype == consts.LIFECYCLE_EVENTS:
-            LOG.info(_LI("Start listening events for cluster (%s)."), cid)
+            LOG.info("Start listening events for cluster (%s).", cid)
             listener = self._add_listener(cid)
             if listener:
                 entry['listener'] = listener
             else:
-                LOG.warning(_LW("Error creating listener for cluster %s"), cid)
+                LOG.warning("Error creating listener for cluster %s", cid)
                 return None
         else:
-            LOG.warning(_LW("Cluster %(id)s check type %(type)s is invalid."),
+            LOG.warning("Cluster %(id)s check type %(type)s is invalid.",
                         {'id': cid, 'type': ctype})
             return None
 
@@ -297,9 +348,9 @@ class HealthManager(service.Service):
         db_registries = objects.HealthRegistry.claim(self.ctx, self.engine_id)
 
         for r in db_registries:
-            # Claiming indicates we claim a health registry which's engine has
-            # die and it updated engine_id to self engine. But we may not start
-            # check alawys.
+            # Claiming indicates we claim a health registry who's engine was
+            # dead, and we will update the health registry's engine_id with
+            # current engine id. But we may not start check always.
             entry = {
                 'cluster_id': r.cluster_id,
                 'check_type': r.check_type,
@@ -308,8 +359,9 @@ class HealthManager(service.Service):
                 'enabled': r.enabled,
             }
 
-            LOG.info(_LI("Loading cluster %s for health monitoring"),
-                     r.cluster_id)
+            LOG.info("Loading cluster %(c)s enabled=%(e)s for "
+                     "health monitoring",
+                     {'c': r.cluster_id, 'e': r.enabled})
 
             if r.enabled:
                 entry = self._start_check(entry)
@@ -328,7 +380,6 @@ class HealthManager(service.Service):
         server = rpc.get_rpc_server(self.target, self)
         server.start()
         self.TG.add_timer(cfg.CONF.periodic_interval, self._dummy_task)
-        self._load_runtime_registry()
 
     def stop(self):
         self.TG.stop_timers()
@@ -344,14 +395,14 @@ class HealthManager(service.Service):
 
     def register_cluster(self, ctx, cluster_id, check_type, interval=None,
                          params=None, enabled=True):
-        r"""Register cluster for health checking.
+        """Register cluster for health checking.
 
         :param ctx: The context of notify request.
         :param cluster_id: The ID of the cluster to be checked.
         :param check_type: A string indicating the type of checks.
         :param interval: An optional integer indicating the length of checking
                          periods in seconds.
-        :param \*\*params: Other parameters for the health check.
+        :param dict params: Other parameters for the health check.
         :return: None
         """
         params = params or {}
@@ -445,14 +496,34 @@ def register(cluster_id, engine_id=None, **kwargs):
                   enabled=enabled)
 
 
-def unregister(cluster_id, engine_id=None):
-    return notify(engine_id, 'unregister_cluster', cluster_id=cluster_id)
+def unregister(cluster_id):
+    engine_id = get_manager_engine(cluster_id)
+    if engine_id:
+        return notify(engine_id, 'unregister_cluster', cluster_id=cluster_id)
+    return True
 
 
 def enable(cluster_id, **kwargs):
-    return notify(None, 'enable_cluster', cluster_id=cluster_id, params=kwargs)
+    engine_id = get_manager_engine(cluster_id)
+    if engine_id:
+        return notify(engine_id, 'enable_cluster', cluster_id=cluster_id,
+                      params=kwargs)
+    return False
 
 
 def disable(cluster_id, **kwargs):
-    return notify(None, 'disable_cluster', cluster_id=cluster_id,
-                  params=kwargs)
+    engine_id = get_manager_engine(cluster_id)
+    if engine_id:
+        return notify(engine_id, 'disable_cluster', cluster_id=cluster_id,
+                      params=kwargs)
+    return False
+
+
+def get_manager_engine(cluster_id):
+    ctx = context.get_admin_context()
+
+    registry = objects.HealthRegistry.get(ctx, cluster_id)
+    if not registry:
+        return None
+
+    return registry.engine_id
