@@ -11,12 +11,13 @@
 # under the License.
 
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 import six
 
 from senlin.common import consts
 from senlin.common import exception as exc
-from senlin.common.i18n import _, _LE
+from senlin.common.i18n import _
 from senlin.common import utils
 from senlin.objects import node as no
 from senlin.profiles import base as pb
@@ -130,7 +131,7 @@ class Node(object):
         """Construct a node from node object.
 
         @param context: the context used for DB operations;
-        @param node: a node object that contains all fields;
+        @param obj: a node object that contains all fields;
         """
         kwargs = {
             'id': obj.id,
@@ -210,7 +211,7 @@ class Node(object):
 
     def do_create(self, context):
         if self.status != consts.NS_INIT:
-            LOG.error(_LE('Node is in status "%s"'), self.status)
+            LOG.error('Node is in status "%s"', self.status)
             return False
 
         self.set_status(context, consts.NS_CREATING, _('Creation in progress'))
@@ -318,30 +319,53 @@ class Node(object):
         if not self.physical_id:
             return False
 
-        res = True
         try:
             res = pb.Profile.check_object(context, self)
         except exc.EResourceOperation as ex:
             self.set_status(context, consts.NS_ERROR, six.text_type(ex))
             return False
 
+        # Physical object is ACTIVE but for some reason the node status in
+        # senlin was WARNING. We only update the status_reason
         if res:
+            if self.status == consts.NS_WARNING:
+                msg = _("Check: Physical object is ACTIVE but the node status "
+                        "was WARNING. %s") % self.status_reason
+                self.set_status(context, consts.NS_WARNING, msg)
+                return True
+
             self.set_status(context, consts.NS_ACTIVE,
                             _("Check: Node is ACTIVE."))
         else:
             self.set_status(context, consts.NS_ERROR,
                             _("Check: Node is not ACTIVE."))
 
-        return res
+        return True
 
-    def do_recover(self, context, **options):
+    def do_recover(self, context, action):
         """recover a node.
 
         This function is supposed to be invoked from a NODE_RECOVER action.
+        :param context: The request context of the action.
+        :param dict options: A map containing the recovery actions (with
+            parameters if any) and fencing settings.
         """
         if not self.physical_id:
             return False
 
+        options = action.inputs
+        if options.get('check', False):
+            res = False
+            try:
+                res = pb.Profile.check_object(context, self)
+            except exc.EResourceOperation:
+                pass
+
+            if res:
+                self.set_status(context, consts.NS_ACTIVE,
+                                reason=_("Recover: Node is ACTIVE."))
+                return True
+
         self.set_status(context, consts.NS_RECOVERING,
                         reason=_('Recover in progress'))
 
@@ -358,6 +382,8 @@ class Node(object):
 
         params = {}
         if self.physical_id != physical_id:
+            self.data['recovery'] = consts.RECOVER_RECREATE
+            params['data'] = self.data
             params['physical_id'] = physical_id
         self.set_status(context, consts.NS_ACTIVE,
                         reason=_('Recover succeeded'), **params)
@@ -390,3 +416,32 @@ class Node(object):
         self.set_status(context, consts.NS_ACTIVE,
                         reason=_("Operation '%s' succeeded") % op)
         return True
+
+    def run_workflow(self, **options):
+        if not self.physical_id:
+            return False
+
+        workflow_name = options.pop('workflow_name')
+        inputs = options.pop('inputs')
+        definition = inputs.pop('definition', None)
+        params = {
+            'cluster_id': self.cluster_id,
+            'node_id': self.physical_id,
+        }
+        params.update(inputs)
+
+        try:
+            profile = self.rt['profile']
+            wfc = profile.workflow(self)
+            workflow = wfc.workflow_find(workflow_name)
+            if workflow is None:
+                wfc.workflow_create(definition, scope="private")
+            else:
+                definition = workflow.definition
+            inputs_str = jsonutils.dumps(params)
+            wfc.execution_create(workflow_name, str(inputs_str))
+        except exc.InternalError as ex:
+            raise exc.EResourceOperation(op='executing', type='workflow',
+                                         id=workflow_name,
+                                         message=six.text_type(ex))
+        return True
