@@ -68,11 +68,12 @@ class Receiver(object):
         self.params = kwargs.get('params', {})
         self.channel = kwargs.get('channel', {})
 
-    def store(self, context):
+    def store(self, context, update=False):
         """Store the receiver in database and return its ID.
 
         :param context: Context for DB operations.
         """
+        timestamp = timeutils.utcnow(True)
         self.created_at = timeutils.utcnow(True)
         values = {
             'id': self.id,
@@ -90,9 +91,15 @@ class Receiver(object):
             'channel': self.channel,
         }
 
-        # TODO(Qiming): Add support to update
-        receiver = ro.Receiver.create(context, values)
-        self.id = receiver.id
+        if update:
+            self.updated_at = timestamp
+            values['updated_at'] = timestamp
+            ro.Receiver.update(context, self.id, values)
+        else:
+            self.created_at = timestamp
+            values['created_at'] = timestamp
+            receiver = ro.Receiver.create(context, values)
+            self.id = receiver.id
 
         return self.id
 
@@ -108,11 +115,11 @@ class Receiver(object):
             # otherwise, use context user
             cdata['trust_id'] = context.trusts
 
-        kwargs['id'] = uuidutils.generate_uuid()
         kwargs['actor'] = cdata
         kwargs['user'] = context.user
         kwargs['project'] = context.project
         kwargs['domain'] = context.domain
+        kwargs['id'] = uuidutils.generate_uuid()
         cluster_id = cluster.id if cluster else None
         obj = cls(rtype, cluster_id, action, **kwargs)
         obj.initialize_channel(context)
