@@ -39,6 +39,13 @@ class ClusterListRequest(base.SenlinObject):
 @base.SenlinObjectRegistry.register
 class ClusterCreateRequestBody(base.SenlinObject):
 
+    # VERSION 1.0: initial version
+    # VERSION 1.1: added field 'config'
+    VERSION = '1.1'
+    VERSION_MAP = {
+        '1.7': '1.1',
+    }
+
     fields = {
         'name': fields.NameField(),
         'profile_id': fields.StringField(),
@@ -49,13 +56,21 @@ class ClusterCreateRequestBody(base.SenlinObject):
             nullable=True, minimum=-1,
             default=consts.CLUSTER_DEFAULT_MAX_SIZE),
         'desired_capacity': fields.CapacityField(
-            nullable=True, minimum=0,
-            default=consts.CLUSTER_DEFAULT_MIN_SIZE),
+            nullable=True, minimum=0),
         'metadata': fields.JsonField(nullable=True, default={}),
         'timeout': fields.NonNegativeIntegerField(
             nullable=True, default=CONF.default_action_timeout),
+        'config': fields.JsonField(nullable=True, default={}),
     }
 
+    def obj_make_compatible(self, primitive, target_version):
+        super(ClusterCreateRequest, self).obj_make_compatible(
+            primitive, target_version)
+        target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version < (1, 1):
+            if 'config' in primitive['senlin_object.data']:
+                del primitive['senlin_object.data']['config']
+
 
 @base.SenlinObjectRegistry.register
 class ClusterCreateRequest(base.SenlinObject):
@@ -76,14 +91,36 @@ class ClusterGetRequest(base.SenlinObject):
 @base.SenlinObjectRegistry.register
 class ClusterUpdateRequest(base.SenlinObject):
 
+    # VERSION 1.0: initial version
+    # VERSION 1.1: added field 'profile_only'
+    # VERSION 1.2: added field 'config'
+    VERSION = '1.2'
+    VERSION_MAP = {
+        '1.6': '1.1',
+        '1.7': '1.2',
+    }
+
     fields = {
         'identity': fields.StringField(),
         'name': fields.NameField(nullable=True),
         'profile_id': fields.StringField(nullable=True),
         'metadata': fields.JsonField(nullable=True),
         'timeout': fields.NonNegativeIntegerField(nullable=True),
+        'profile_only': fields.BooleanField(nullable=True),
+        'config': fields.JsonField(nullable=True),
     }
 
+    def obj_make_compatible(self, primitive, target_version):
+        super(ClusterUpdateRequest, self).obj_make_compatible(
+            primitive, target_version)
+        target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version < (1, 1):
+            if 'profile_only' in primitive['senlin_object.data']:
+                del primitive['senlin_object.data']['profile_only']
+        if target_version < (1, 2):
+            if 'config' in primitive['senlin_object.data']:
+                del primitive['senlin_object.data']['config']
+
 
 @base.SenlinObjectRegistry.register
 class ClusterAddNodesRequest(base.SenlinObject):
@@ -114,11 +151,10 @@ class ClusterDelNodesRequest(base.SenlinObject):
     def obj_make_compatible(self, primitive, target_version):
         super(ClusterDelNodesRequest, self).obj_make_compatible(
             primitive, target_version)
-        target_version = versionutils.convert_version_to_tuple(
-            target_version)
+        target_version = versionutils.convert_version_to_tuple(target_version)
         if target_version < (1, 1):
-            if 'destroy_after_deletion' in primitive:
-                del primitive['destroy_after_deletion']
+            if 'destroy_after_deletion' in primitive['senlin_object.data']:
+                del primitive['senlin_object.data']['destroy_after_deletion']
 
 
 @base.SenlinObjectRegistry.register
@@ -140,7 +176,7 @@ class ClusterResizeRequest(base.SenlinObject):
         'min_size': fields.CapacityField(nullable=True, minimum=0),
         'max_size': fields.CapacityField(nullable=True, minimum=-1),
         'min_step': fields.NonNegativeIntegerField(nullable=True),
-        'strict': fields.BooleanField(nullable=True, default=True)
+        'strict': fields.BooleanField(nullable=True, default=True),
     }
 
 
@@ -149,7 +185,7 @@ class ClusterScaleInRequest(base.SenlinObject):
 
     fields = {
         'identity': fields.StringField(),
-        'count': fields.NonNegativeIntegerField(nullable=True)
+        'count': fields.NonNegativeIntegerField(nullable=True),
     }
 
 
@@ -158,7 +194,7 @@ class ClusterScaleOutRequest(base.SenlinObject):
 
     fields = {
         'identity': fields.StringField(),
-        'count': fields.NonNegativeIntegerField(nullable=True)
+        'count': fields.NonNegativeIntegerField(nullable=True),
     }
 
 
