@@ -14,7 +14,7 @@
 Policy for load-balancing among nodes in a cluster.
 
 NOTE: For full documentation about how the deletion policy works, check:
-http://docs.openstack.org/developer/senlin/developer/policies/
+https://docs.openstack.org/senlin/latest/developer/policies/
 load_balance_v1.html
 """
 
@@ -24,12 +24,10 @@ from oslo_log import log as logging
 from senlin.common import constraints
 from senlin.common import consts
 from senlin.common import exception as exc
-from senlin.common.i18n import _, _LW
+from senlin.common.i18n import _
 from senlin.common import scaleutils
 from senlin.common import schema
 from senlin.engine import cluster_policy
-from senlin.engine import node as nm
-from senlin.objects import cluster as co
 from senlin.objects import node as no
 from senlin.policies import base
 
@@ -57,6 +55,7 @@ class LoadBalancingPolicy(base.Policy):
         ('AFTER', consts.CLUSTER_ADD_NODES),
         ('AFTER', consts.CLUSTER_SCALE_OUT),
         ('AFTER', consts.CLUSTER_RESIZE),
+        ('AFTER', consts.NODE_RECOVER),
         ('AFTER', consts.NODE_CREATE),
         ('BEFORE', consts.CLUSTER_DEL_NODES),
         ('BEFORE', consts.CLUSTER_SCALE_IN),
@@ -262,11 +261,9 @@ class LoadBalancingPolicy(base.Policy):
             },
         ),
         LB_STATUS_TIMEOUT: schema.Integer(
-            _('Time in second to wait for loadbalancer to be ready'
-              '(provisioning_status is ACTIVE and operating_status is '
-              'ONLINE) before and after senlin requests lbaas V2 service '
-              'for lb operations. '),
-            default=600,
+            _('Time in second to wait for loadbalancer to become ready '
+              'after senlin requests LBaaS V2 service for operations.'),
+            default=300,
         )
     }
 
@@ -317,9 +314,6 @@ class LoadBalancingPolicy(base.Policy):
         if res is False:
             return False, data
 
-        nodes = nm.Node.load_all(oslo_context.get_current(),
-                                 cluster_id=cluster.id)
-
         lb_driver = self.lbaas(cluster.user, cluster.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
 
@@ -334,7 +328,7 @@ class LoadBalancingPolicy(base.Policy):
         port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
         subnet = self.pool_spec.get(self.POOL_SUBNET)
 
-        for node in nodes:
+        for node in cluster.nodes:
             member_id = lb_driver.member_add(node, data['loadbalancer'],
                                              data['pool'], port, subnet)
             if member_id is None:
@@ -346,7 +340,8 @@ class LoadBalancingPolicy(base.Policy):
                 return False, 'Failed in adding node into lb pool'
 
             node.data.update({'lb_member': member_id})
-            node.store(oslo_context.get_current())
+            values = {'data': node.data}
+            no.Node.update(oslo_context.get_current(), node.id, values)
 
         cluster_data_lb = cluster.data.get('loadbalancers', {})
         cluster_data_lb[self.id] = {'vip_address': data.pop('vip_address')}
@@ -380,12 +375,11 @@ class LoadBalancingPolicy(base.Policy):
         if res is False:
             return False, reason
 
-        nodes = nm.Node.load_all(oslo_context.get_current(),
-                                 cluster_id=cluster.id, project_safe=False)
-        for node in nodes:
+        for node in cluster.nodes:
             if 'lb_member' in node.data:
                 node.data.pop('lb_member')
-                node.store(oslo_context.get_current())
+                values = {'data': node.data}
+                no.Node.update(oslo_context.get_current(), node.id, values)
 
         lb_data = cluster.data.get('loadbalancers', {})
         if lb_data and isinstance(lb_data, dict):
@@ -404,7 +398,7 @@ class LoadBalancingPolicy(base.Policy):
         candidates = None
         if deletion is None:
             if action.action == consts.NODE_DELETE:
-                candidates = [action.node.id]
+                candidates = [action.entity.id]
                 count = 1
             elif action.action == consts.CLUSTER_DEL_NODES:
                 # Get candidates from action.input
@@ -412,9 +406,9 @@ class LoadBalancingPolicy(base.Policy):
                 count = len(candidates)
             elif action.action == consts.CLUSTER_RESIZE:
                 # Calculate deletion count based on action input
-                db_cluster = co.Cluster.get(action.context, cluster_id)
-                current = no.Node.count_by_cluster(action.context, cluster_id)
-                scaleutils.parse_resize_params(action, db_cluster, current)
+                cluster = action.entity
+                current = len(cluster.nodes)
+                scaleutils.parse_resize_params(action, cluster, current)
                 if 'deletion' not in action.data:
                     return []
                 else:
@@ -426,10 +420,11 @@ class LoadBalancingPolicy(base.Policy):
             candidates = deletion.get('candidates', None)
 
         # Still no candidates available, pick count of nodes randomly
+        # apply to CLUSTER_RESIZE/CLUSTER_SCALE_IN
         if candidates is None:
             if count == 0:
                 return []
-            nodes = no.Node.get_all_by_cluster(action.context, cluster_id)
+            nodes = action.entity.nodes
             if count > len(nodes):
                 count = len(nodes)
             candidates = scaleutils.nodes_by_random(nodes, count)
@@ -442,6 +437,106 @@ class LoadBalancingPolicy(base.Policy):
 
         return candidates
 
+    def _remove_member(self, candidates, policy, action, driver,
+                       handle_err=True):
+        # Load policy data
+        policy_data = self._extract_policy_data(policy.data)
+        lb_id = policy_data['loadbalancer']
+        pool_id = policy_data['pool']
+
+        failed_nodes = []
+        for node_id in candidates:
+            node = no.Node.get(action.context, node_id=node_id)
+            node_data = node.data or {}
+            member_id = node_data.get('lb_member', None)
+            if member_id is None:
+                LOG.warning('Node %(n)s not found in lb pool %(p)s.',
+                            {'n': node_id, 'p': pool_id})
+                continue
+
+            res = driver.member_remove(lb_id, pool_id, member_id)
+            values = {}
+            if res is not True and handle_err is True:
+                failed_nodes.append(node.id)
+                values['status'] = consts.NS_WARNING
+                values['status_reason'] = _(
+                    'Failed in removing node from lb pool.')
+            else:
+                node.data.pop('lb_member', None)
+                values['data'] = node.data
+            no.Node.update(action.context, node_id, values)
+
+        return failed_nodes
+
+    def _add_member(self, candidates, policy, action, driver):
+        # Load policy data
+        policy_data = self._extract_policy_data(policy.data)
+        lb_id = policy_data['loadbalancer']
+        pool_id = policy_data['pool']
+        port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
+        subnet = self.pool_spec.get(self.POOL_SUBNET)
+
+        failed_nodes = []
+        for node_id in candidates:
+            node = no.Node.get(action.context, node_id=node_id)
+            node_data = node.data or {}
+            member_id = node_data.get('lb_member', None)
+            if member_id:
+                LOG.warning('Node %(n)s already in lb pool %(p)s.',
+                            {'n': node_id, 'p': pool_id})
+                continue
+
+            member_id = driver.member_add(node, lb_id, pool_id, port, subnet)
+            values = {}
+            if member_id is None:
+                failed_nodes.append(node.id)
+                values['status'] = consts.NS_WARNING
+                values['status_reason'] = _(
+                    'Failed in adding node into lb pool.')
+            else:
+                node.data.update({'lb_member': member_id})
+                values['data'] = node.data
+            no.Node.update(action.context, node_id, values)
+
+        return failed_nodes
+
+    def _get_post_candidates(self, action):
+        # This method will parse action data passed from action layer
+        candidates = []
+        if (action.action == consts.NODE_CREATE or
+                action.action == consts.NODE_RECOVER):
+            candidates = [action.entity.id]
+        else:
+            creation = action.data.get('creation', None)
+            candidates = creation.get('nodes', []) if creation else []
+
+        return candidates
+
+    def _process_recovery(self, candidates, policy, driver, action):
+        # Process node recovery action
+        node = action.entity
+        data = node.data
+        lb_member = data.get('lb_member', None)
+        recovery = data.pop('recovery', None)
+        values = {}
+
+        # lb_member is None, need to add to lb pool
+        if not lb_member:
+            values['data'] = data
+            no.Node.update(action.context, values)
+            return candidates
+
+        # was a member of lb pool, check whether has been recreated
+        if recovery is not None and recovery == consts.RECOVER_RECREATE:
+            self._remove_member(candidates, policy, action, driver,
+                                handle_err=False)
+            data.pop('lb_member', None)
+            values['data'] = data
+            no.Node.update(action.context, values)
+            return candidates
+
+        return None
+
     def pre_op(self, cluster_id, action):
         """Routine to be called before an action has been executed.
 
@@ -458,30 +553,20 @@ class LoadBalancingPolicy(base.Policy):
         if len(candidates) == 0:
             return
 
-        db_cluster = co.Cluster.get(action.context, cluster_id)
-        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        obj = action.entity
+        lb_driver = self.lbaas(obj.user, obj.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
         cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
                                                self.id)
-        policy_data = self._extract_policy_data(cp.data)
-        lb_id = policy_data['loadbalancer']
-        pool_id = policy_data['pool']
 
         # Remove nodes that will be deleted from lb pool
-        for node_id in candidates:
-            node = nm.Node.load(action.context, node_id=node_id)
-            member_id = node.data.get('lb_member', None)
-            if member_id is None:
-                LOG.warning(_LW('Node %(n)s not found in lb pool %(p)s.'),
-                            {'n': node_id, 'p': pool_id})
-                continue
+        failed_nodes = self._remove_member(candidates, cp, action, lb_driver)
 
-            res = lb_driver.member_remove(lb_id, pool_id, member_id)
-            if res is not True:
-                action.data['status'] = base.CHECK_ERROR
-                action.data['reason'] = _('Failed in removing deleted '
-                                          'node(s) from lb pool.')
-                return
+        if failed_nodes:
+            error = _('Failed in removing deleted node(s) from lb pool: %s'
+                      ) % failed_nodes
+            action.data['status'] = base.CHECK_ERROR
+            action.data['reason'] = error
 
         return
 
@@ -498,43 +583,26 @@ class LoadBalancingPolicy(base.Policy):
         """
         # TODO(Yanyanhu): Need special handling for cross-az scenario
         # which is supported by Neutron lbaas.
-        if action.action == consts.NODE_CREATE:
-            nodes_added = [action.node.id]
-        else:
-            creation = action.data.get('creation', None)
-            nodes_added = creation.get('nodes', []) if creation else []
-            if len(nodes_added) == 0:
-                return
+        candidates = self._get_post_candidates(action)
+        if len(candidates) == 0:
+            return
 
-        db_cluster = co.Cluster.get(action.context, cluster_id)
-        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        obj = action.entity
+        lb_driver = self.lbaas(obj.user, obj.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
         cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
                                                self.id)
-        policy_data = self._extract_policy_data(cp.data)
-        lb_id = policy_data['loadbalancer']
-        pool_id = policy_data['pool']
-        port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
-        subnet = self.pool_spec.get(self.POOL_SUBNET)
-
-        # Add new nodes to lb pool
-        for node_id in nodes_added:
-            node = nm.Node.load(action.context, node_id=node_id)
-            member_id = node.data.get('lb_member', None)
-            if member_id:
-                LOG.warning(_LW('Node %(n)s already in lb pool %(p)s.'),
-                            {'n': node_id, 'p': pool_id})
-                continue
-
-            member_id = lb_driver.member_add(node, lb_id, pool_id, port,
-                                             subnet)
-            if member_id is None:
-                action.data['status'] = base.CHECK_ERROR
-                action.data['reason'] = _('Failed in adding new node(s) '
-                                          'into lb pool.')
+        if action.action == consts.NODE_RECOVER:
+            candidates = self._process_recovery(
+                candidates, cp, lb_driver, action)
+            if not candidates:
                 return
 
-            node.data.update({'lb_member': member_id})
-            node.store(action.context)
+        # Add new nodes to lb pool
+        failed_nodes = self._add_member(candidates, cp, action, lb_driver)
+        if failed_nodes:
+            error = _('Failed in adding nodes into lb pool: %s') % failed_nodes
+            action.data['status'] = base.CHECK_ERROR
+            action.data['reason'] = error
 
         return
