@@ -22,7 +22,7 @@ import six
 from senlin.common import consts
 from senlin.common import context
 from senlin.common import exception as exc
-from senlin.common.i18n import _, _LE, _LW
+from senlin.common.i18n import _
 from senlin.common import schema
 from senlin.common import utils
 from senlin.drivers import base as driver_base
@@ -125,6 +125,8 @@ class Profile(object):
         self._computeclient = None
         self._networkclient = None
         self._orchestrationclient = None
+        self._workflowclient = None
+        self._block_storageclient = None
 
     @classmethod
     def _from_object(cls, profile):
@@ -171,7 +173,6 @@ class Profile(object):
         if metadata is None:
             metadata = {}
 
-        profile = None
         try:
             profile = cls(name, spec, metadata=metadata, user=ctx.user,
                           project=ctx.project)
@@ -243,6 +244,25 @@ class Profile(object):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_get_details(obj)
 
+    @classmethod
+    @profiler.trace('Profile.adopt_node', hide_args=False)
+    def adopt_node(cls, ctx, obj, type_name, overrides=None, snapshot=False):
+        """Adopt a node.
+
+        :param ctx: Request context.
+        :param obj: A temporary node object.
+        :param overrides: An optional parameter that specifies the set of
+            properties to be overridden.
+        :param snapshot: A boolean flag indicating whether a snapshot should
+            be created before adopting the node.
+        :returns: A dictionary containing the profile spec created from the
+            specific node, or a dictionary containing error message.
+        """
+        parts = type_name.split("-")
+        tmpspec = {"type": parts[0], "version": parts[1]}
+        profile = cls("name", tmpspec)
+        return profile.do_adopt(obj, overrides=overrides, snapshot=snapshot)
+
     @classmethod
     @profiler.trace('Profile.join_cluster', hide_args=False)
     def join_cluster(cls, ctx, obj, cluster_id):
@@ -259,7 +279,11 @@ class Profile(object):
     @profiler.trace('Profile.check_object', hide_args=False)
     def check_object(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
-        return profile.do_check(obj)
+        try:
+            return profile.do_check(obj)
+        except exc.InternalError as ex:
+            LOG.error(ex)
+            return False
 
     @classmethod
     @profiler.trace('Profile.recover_object', hide_args=False)
@@ -368,6 +392,27 @@ class Profile(object):
         self._orchestrationclient = oc
         return oc
 
+    def workflow(self, obj):
+        if self._workflowclient is not None:
+            return self._workflowclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._workflowclient = driver_base.SenlinDriver().workflow(params)
+        return self._workflowclient
+
+    def block_storage(self, obj):
+        """Construct cinder client based on object.
+
+        :param obj: Object for which the client is created. It is expected to
+                    be None when retrieving an existing client. When creating
+                    a client, it contains the user and project to be used.
+        """
+        if self._block_storageclient is not None:
+            return self._block_storageclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._block_storageclient = driver_base.SenlinDriver().block_storage(
+            params)
+        return self._block_storageclient
+
     def do_create(self, obj):
         """For subclass to override."""
         raise NotImplementedError
@@ -378,46 +423,57 @@ class Profile(object):
 
     def do_update(self, obj, new_profile, **params):
         """For subclass to override."""
-        LOG.warning(_LW("Update operation not supported."))
+        LOG.warning("Update operation not supported.")
         return True
 
     def do_check(self, obj):
         """For subclass to override."""
-        LOG.warning(_LW("Check operation not supported."))
+        LOG.warning("Check operation not supported.")
         return True
 
     def do_get_details(self, obj):
         """For subclass to override."""
-        LOG.warning(_LW("Get_details operation not supported."))
+        LOG.warning("Get_details operation not supported.")
+        return {}
+
+    def do_adopt(self, obj, overrides=None, snapshot=False):
+        """For subclass to override."""
+        LOG.warning("Adopt operation not supported.")
         return {}
 
     def do_join(self, obj, cluster_id):
         """For subclass to override to perform extra operations."""
-        LOG.warning(_LW("Join operation not specialized."))
+        LOG.warning("Join operation not specialized.")
         return True
 
     def do_leave(self, obj):
         """For subclass to override to perform extra operations."""
-        LOG.warning(_LW("Leave operation not specialized."))
+        LOG.warning("Leave operation not specialized.")
         return True
 
     def do_recover(self, obj, **options):
         """Default recover operation.
 
+        This is provided as a fallback if a specific profile type does not
+        override this method.
+
         :param obj: The node object to operate on.
         :param options: Keyword arguments for the recover operation.
         """
         operation = options.pop('operation', None)
-        # TODO(Qiming): The operation input could be a list of operations.
+
+        # The operation is a list of action names with optional parameters
         if operation and not isinstance(operation, six.string_types):
             operation = operation[0]
 
-        if operation and operation != consts.RECOVER_RECREATE:
-            LOG.error(_LE("Recover operation not supported: %s"), operation)
+        if operation and operation['name'] != consts.RECOVER_RECREATE:
+            LOG.error("Recover operation not supported: %s", operation)
             return False
 
+        extra_params = options.get('params', {})
+        fence_compute = extra_params.get('fence_compute', False)
         try:
-            self.do_delete(obj, **options)
+            self.do_delete(obj, force=fence_compute)
         except exc.EResourceDeletion as ex:
             raise exc.EResourceOperation(op='recovering', type='node',
                                          id=obj.id, message=six.text_type(ex))
@@ -431,7 +487,7 @@ class Profile(object):
 
     def do_validate(self, obj):
         """For subclass to override."""
-        LOG.warning(_LW("Validate operation not supported."))
+        LOG.warning("Validate operation not supported.")
         return True
 
     def to_dict(self):
@@ -460,6 +516,5 @@ class Profile(object):
             return True
 
         msg = ", ".join(non_updatables)
-        LOG.error(_LE("The following properties are not updatable: %s."
-                      ) % msg)
+        LOG.error("The following properties are not updatable: %s.", msg)
         return False
