@@ -13,6 +13,7 @@
 import base64
 import copy
 
+from oslo_log import log as logging
 from oslo_utils import encodeutils
 import six
 
@@ -25,6 +26,8 @@ from senlin.common import schema
 from senlin.objects import node as node_obj
 from senlin.profiles import base
 
+LOG = logging.getLogger(__name__)
+
 
 class ServerProfile(base.Profile):
     """Profile for an OpenStack Nova server."""
@@ -61,9 +64,11 @@ class ServerProfile(base.Profile):
     )
 
     NETWORK_KEYS = (
-        PORT, FIXED_IP, NETWORK,
+        PORT, FIXED_IP, NETWORK, PORT_SECURITY_GROUPS,
+        FLOATING_NETWORK, FLOATING_IP,
     ) = (
-        'port', 'fixed_ip', 'network',
+        'port', 'fixed_ip', 'network', 'security_groups',
+        'floating_network', 'floating_ip',
     )
 
     PERSONALITY_KEYS = (
@@ -103,13 +108,13 @@ class ServerProfile(base.Profile):
                         _('ID of the source image, snapshot or volume'),
                     ),
                     BDM2_SOURCE_TYPE: schema.String(
-                        _('Volume source type, should be image, snapshot, '
-                          'volume or blank'),
+                        _("Volume source type, must be one of 'image', "
+                          "'snapshot', 'volume' or 'blank'"),
                         required=True,
                     ),
                     BDM2_DESTINATION_TYPE: schema.String(
-                        _('Volume destination type, should be volume or '
-                          'local'),
+                        _("Volume destination type, must be 'volume' or "
+                          "'local'"),
                         required=True,
                     ),
                     BDM2_DISK_BUS: schema.String(
@@ -159,7 +164,7 @@ class ServerProfile(base.Profile):
         ),
         METADATA: schema.Map(
             _('A collection of key/value pairs to be associated with the '
-              'server created. Both key and value should be <=255 chars.'),
+              'server created. Both key and value must be <=255 chars.'),
             updatable=True,
         ),
         NAME: schema.String(
@@ -180,6 +185,21 @@ class ServerProfile(base.Profile):
                     FIXED_IP: schema.String(
                         _('Fixed IP to be used by the network.'),
                     ),
+                    PORT_SECURITY_GROUPS: schema.List(
+                        _('A list of security groups to be attached to '
+                          'this port.'),
+                        schema=schema.String(
+                            _('Name of a security group'),
+                            required=True,
+                        ),
+                    ),
+                    FLOATING_NETWORK: schema.String(
+                        _('The nework on which to create a floating IP'),
+                    ),
+                    FLOATING_IP: schema.String(
+                        _('The floating IP address to be associated with '
+                          'this port.'),
+                    ),
                 },
             ),
             updatable=True,
@@ -203,7 +223,7 @@ class ServerProfile(base.Profile):
         ),
         SCHEDULER_HINTS: schema.Map(
             _('A collection of key/value pairs to be associated with the '
-              'Scheduler hints. Both key and value should be <=255 chars.'),
+              'Scheduler hints. Both key and value must be <=255 chars.'),
         ),
 
         SECURITY_GROUPS: schema.List(
@@ -341,7 +361,7 @@ class ServerProfile(base.Profile):
             flavor = self.compute(obj).flavor_find(name_or_id, False)
         except exc.InternalError as ex:
             msg = six.text_type(ex)
-            if reason is None:  # reaons is 'validate'
+            if reason is None:  # reason is 'validate'
                 if ex.code == 404:
                     msg = _("The specified %(k)s '%(v)s' could not be found."
                             ) % {'k': self.FLAVOR, 'v': name_or_id}
@@ -435,53 +455,109 @@ class ServerProfile(base.Profile):
                     del bd[key]
         return bdm
 
-    def _validate_network(self, obj, network, reason=None):
-        result = {}
-        error = None
-        # check network
-        net_ident = network.get(self.NETWORK)
-        if net_ident:
+    def _check_security_groups(self, nc, net_spec, result):
+        """Check security groups.
+
+        :param nc: network driver connection.
+        :param net_spec: the specification to check.
+        :param result: the result that is used as return value.
+        :returns: None if succeeded or an error message if things go wrong.
+        """
+        sgs = net_spec.get(self.PORT_SECURITY_GROUPS)
+        if not sgs:
+            return
+
+        res = []
+        try:
+            for sg in sgs:
+                sg_obj = nc.security_group_find(sg)
+                res.append(sg_obj.id)
+        except exc.InternalError as ex:
+            return six.text_type(ex)
+
+        result[self.PORT_SECURITY_GROUPS] = res
+        return
+
+    def _check_network(self, nc, net, result):
+        """Check the specified network.
+
+        :param nc: network driver connection.
+        :param net: the name or ID of network to check.
+        :param result: the result that is used as return value.
+        :returns: None if succeeded or an error message if things go wrong.
+        """
+        if net is None:
+            return
+        try:
+            net_obj = nc.network_get(net)
+            result[self.NETWORK] = net_obj.id
+        except exc.InternalError as ex:
+            return six.text_type(ex)
+
+    def _check_port(self, nc, port, result):
+        """Check the specified port.
+
+        :param nc: network driver connection.
+        :param port: the name or ID of port to check.
+        :param result: the result that is used as return value.
+        :returns: None if succeeded or an error message if things go wrong.
+        """
+        if port is None:
+            return
+
+        try:
+            port_obj = nc.port_find(port)
+            if port_obj.status != 'DOWN':
+                return _("The status of the port %(p)s must be DOWN"
+                         ) % {'p': port}
+            result[self.PORT] = port_obj.id
+            return
+        except exc.InternalError as ex:
+            return six.text_type(ex)
+
+    def _check_floating_ip(self, nc, net_spec, result):
+        """Check floating IP and network, if specified.
+
+        :param nc: network driver connection.
+        :param net_spec: the specification to check.
+        :param result: the result that is used as return value.
+        :returns: None if succeeded or an error message if things go wrong.
+        """
+        net = net_spec.get(self.FLOATING_NETWORK)
+        if net:
             try:
-                net = self.network(obj).network_get(net_ident)
-                if reason == 'update':
-                    result['net_id'] = net.id
-                else:
-                    result['uuid'] = net.id
+                net_obj = nc.network_get(net)
+                result[self.FLOATING_NETWORK] = net_obj.id
             except exc.InternalError as ex:
-                error = six.text_type(ex)
+                return six.text_type(ex)
 
-        # check port
-        port_ident = network.get(self.PORT)
-        if not error and port_ident:
-            try:
-                port = self.network(obj).port_find(port_ident)
-                if port.status != 'DOWN':
-                    error = _("The status of the port %(port)s must be DOWN"
-                              ) % {'port': port_ident}
+        flt_ip = net_spec.get(self.FLOATING_IP)
+        if not flt_ip:
+            return
 
-                if reason == 'update':
-                    result['port_id'] = port.id
-                else:
-                    result['port'] = port.id
-            except exc.InternalError as ex:
-                error = six.text_type(ex)
-        elif port_ident is None and net_ident is None:
-            error = _("'%(port)s' is required if '%(net)s' is omitted"
-                      ) % {'port': self.PORT, 'net': self.NETWORK}
+        try:
+            # Find floating ip with this address
+            fip = nc.floatingip_find(flt_ip)
+            if fip:
+                if fip.status == 'ACTIVE':
+                    return _('the floating IP %s has been used.') % flt_ip
+                result['floating_ip_id'] = fip.id
 
-        fixed_ip = network.get(self.FIXED_IP)
-        if not error and fixed_ip:
-            if port_ident is not None:
-                error = _("The '%(port)s' property and the '%(fixed_ip)s' "
-                          "property cannot be specified at the same time"
-                          ) % {'port': self.PORT, 'fixed_ip': self.FIXED_IP}
-            else:
-                if reason == 'update':
-                    result['fixed_ips'] = [{'ip_address': fixed_ip}]
-                else:
-                    result['fixed_ip'] = fixed_ip
+            # Create a floating IP with address if floating ip unspecified
+            if not net:
+                return _('Must specify a network to create floating IP')
+
+            result[self.FLOATING_IP] = flt_ip
+            return
+        except exc.InternalError as ex:
+            return six.text_type(ex)
+
+    def _validate_network(self, obj, net_spec, reason=None):
+
+        def _verify(error):
+            if error is None:
+                return
 
-        if error:
             if reason == 'create':
                 raise exc.EResourceCreation(type='server', message=error)
             elif reason == 'update':
@@ -490,8 +566,182 @@ class ServerProfile(base.Profile):
             else:
                 raise exc.InvalidSpec(message=error)
 
+        nc = self.network(obj)
+        result = {}
+
+        # check network
+        net = net_spec.get(self.NETWORK)
+        error = self._check_network(nc, net, result)
+        _verify(error)
+
+        # check port
+        port = net_spec.get(self.PORT)
+        error = self._check_port(nc, port, result)
+        _verify(error)
+
+        if port is None and net is None:
+            _verify(_("One of '%(p)s' and '%(n)s' must be provided"
+                      ) % {'p': self.PORT, 'n': self.NETWORK})
+
+        fixed_ip = net_spec.get(self.FIXED_IP)
+        if fixed_ip:
+            if port is not None:
+                _verify(_("The '%(p)s' property and the '%(fip)s' property "
+                          "cannot be specified at the same time"
+                          ) % {'p': self.PORT, 'fip': self.FIXED_IP})
+            result[self.FIXED_IP] = fixed_ip
+
+        # Check security_groups
+        error = self._check_security_groups(nc, net_spec, result)
+        _verify(error)
+
+        # Check floating IP
+        error = self._check_floating_ip(nc, net_spec, result)
+        _verify(error)
+
         return result
 
+    def _get_port(self, obj, net_spec):
+        """Fetch or create a port.
+
+        :param obj: The node object.
+        :param net_spec: The parameters to create a port.
+        :returns: Created port object and error message.
+        """
+        port_id = net_spec.get(self.PORT, None)
+        if port_id:
+            try:
+                port = self.network(obj).port_find(port_id)
+                return port, None
+            except exc.InternalError as ex:
+                return None, ex
+        port_attr = {
+            'network_id': net_spec.get(self.NETWORK),
+        }
+        fixed_ip = net_spec.get(self.FIXED_IP, None)
+        if fixed_ip:
+            port_attr['fixed_ips'] = [fixed_ip]
+        security_groups = net_spec.get(self.PORT_SECURITY_GROUPS, [])
+        if security_groups:
+            port_attr['security_groups'] = security_groups
+        try:
+            port = self.network(obj).port_create(**port_attr)
+            return port, None
+        except exc.InternalError as ex:
+            return None, ex
+
+    def _delete_ports(self, obj, ports):
+        """Delete ports.
+
+        :param obj: The node object
+        :param ports: A list of internal ports.
+        :returns: None for succeed or error for failure.
+        """
+        pp = copy.deepcopy(ports)
+        for port in pp:
+            # remove port created by senlin
+            if port.get('remove', False):
+                try:
+                    # remove floating IP created by senlin
+                    if port.get('floating', None) and port[
+                            'floating'].get('remove', False):
+                        self.network(obj).floatingip_delete(
+                            port['floating']['id'])
+                    self.network(obj).port_delete(port['id'])
+                except exc.InternalError as ex:
+                    return ex
+                ports.remove(port)
+        node_data = obj.data
+        node_data['internal_ports'] = ports
+        node_obj.Node.update(self.context, obj.id, {'data': node_data})
+
+    def _get_floating_ip(self, obj, fip_spec, port_id):
+        """Find or Create a floating IP.
+
+        :param obj: The node object.
+        :param fip_spec: The parameters to create a floating ip
+        :param port_id: The port ID to associate with
+        :returns: A floating IP object and error message.
+        """
+        floating_ip_id = fip_spec.get('floating_ip_id', None)
+        if floating_ip_id:
+            try:
+                fip = self.network(obj).floatingip_find(floating_ip_id)
+                if fip.port_id is None:
+                    attr = {'port_id': port_id}
+                    fip = self.network(obj).floatingip_update(fip, **attr)
+                return fip, None
+            except exc.InternalError as ex:
+                return None, ex
+        net_id = fip_spec.get(self.FLOATING_NETWORK)
+        fip_addr = fip_spec.get(self.FLOATING_IP)
+        attr = {
+            'port_id': port_id,
+            'floating_network_id': net_id,
+        }
+        if fip_addr:
+            attr.update({'floating_ip_address': fip_addr})
+        try:
+            fip = self.network(obj).floatingip_create(**attr)
+            return fip, None
+        except exc.InternalError as ex:
+            return None, ex
+
+    def _create_ports_from_properties(self, obj, networks, action_type):
+        """Create or find ports based on networks property.
+
+        :param obj: The node object.
+        :param networks: The networks property used for node.
+        :param action_type: Either 'create' or 'update'.
+
+        :returns: A list of created port's attributes.
+        """
+        internal_ports = obj.data.get('internal_ports', [])
+        if not networks:
+            return []
+
+        for net_spec in networks:
+            net = self._validate_network(obj, net_spec, action_type)
+            # Create port
+            port, ex = self._get_port(obj, net)
+            # Delete created ports before raise error
+            if ex:
+                d_ex = self._delete_ports(obj, internal_ports)
+                if d_ex:
+                    raise d_ex
+                else:
+                    raise ex
+            port_attrs = {
+                'id': port.id,
+                'network_id': port.network_id,
+                'security_group_ids': port.security_group_ids,
+                'fixed_ips': port.fixed_ips
+            }
+            if self.PORT not in net:
+                port_attrs.update({'remove': True})
+            # Create floating ip
+            if 'floating_ip_id' in net or self.FLOATING_NETWORK in net:
+                fip, ex = self._get_floating_ip(obj, net, port_attrs['id'])
+                if ex:
+                    d_ex = self._delete_ports(obj, internal_ports)
+                    if d_ex:
+                        raise d_ex
+                    else:
+                        raise ex
+                port_attrs['floating'] = {
+                    'id': fip.id,
+                    'floating_ip_address': fip.floating_ip_address,
+                    'floating_network_id': fip.floating_network_id,
+                }
+                if self.FLOATING_NETWORK in net:
+                    port_attrs['floating'].update({'remove': True})
+            internal_ports.append(port_attrs)
+        if internal_ports:
+            node_data = obj.data
+            node_data.update(internal_ports=internal_ports)
+            node_obj.Node.update(self.context, obj.id, {'data': node_data})
+        return internal_ports
+
     def _build_metadata(self, obj, usermeta):
         """Build custom metadata for server.
 
@@ -579,10 +829,10 @@ class ServerProfile(base.Profile):
 
         networks = self.properties[self.NETWORKS]
         if networks is not None:
-            kwargs['networks'] = []
-            for net_spec in networks:
-                net = self._validate_network(obj, net_spec, 'create')
-                kwargs['networks'].append(net)
+            ports = self._create_ports_from_properties(
+                obj, networks, 'create')
+            kwargs['networks'] = [
+                {'port': port['id']} for port in ports]
 
         secgroups = self.properties[self.SECURITY_GROUPS]
         if secgroups:
@@ -603,6 +853,7 @@ class ServerProfile(base.Profile):
         try:
             server = self.compute(obj).server_create(**kwargs)
             self.compute(obj).wait_for_server(server.id)
+            server = self.compute(obj).server_get(server.id)
             # Update zone placement info if available
             self._update_zone_info(obj, server)
             return server.id
@@ -627,6 +878,7 @@ class ServerProfile(base.Profile):
 
         server_id = obj.physical_id
         ignore_missing = params.get('ignore_missing', True)
+        internal_ports = obj.data.get('internal_ports', [])
         force = params.get('force', False)
 
         try:
@@ -636,6 +888,10 @@ class ServerProfile(base.Profile):
             else:
                 driver.server_delete(server_id, ignore_missing)
             driver.wait_for_server_delete(server_id)
+            if internal_ports:
+                ex = self._delete_ports(obj, internal_ports)
+                if ex:
+                    raise ex
             return True
         except exc.InternalError as ex:
             raise exc.EResourceDeletion(type='server', id=server_id,
@@ -808,7 +1064,7 @@ class ServerProfile(base.Profile):
                                       message=six.text_type(ex))
         return True
 
-    def _create_interfaces(self, obj, networks):
+    def _update_network_add_port(self, obj, networks):
         """Create new interfaces for the server node.
 
         :param obj: The node object to operate.
@@ -824,17 +1080,60 @@ class ServerProfile(base.Profile):
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
                                       message=six.text_type(ex))
 
-        for net_spec in networks:
-            net_attrs = self._validate_network(obj, net_spec, 'update')
-            if net_attrs:
-                try:
-                    cc.server_interface_create(server, **net_attrs)
-                except exc.InternalError as ex:
-                    raise exc.EResourceUpdate(type='server',
-                                              id=obj.physical_id,
-                                              message=six.text_type(ex))
+        ports = self._create_ports_from_properties(
+            obj, networks, 'update')
+        for port in ports:
+            params = {'port': port['id']}
+            try:
+                cc.server_interface_create(server, **params)
+            except exc.InternalError as ex:
+                raise exc.EResourceUpdate(type='server',
+                                          id=obj.physical_id,
+                                          message=six.text_type(ex))
 
-    def _delete_interfaces(self, obj, networks):
+    def _find_port_by_net_spec(self, obj, net_spec, ports):
+        """Find existing ports match with specific network properties.
+
+        :param obj: The node object.
+        :param net_spec: Network property of this profile.
+        :param ports: A list of ports which attached to this server.
+        :returns: A list of candidate ports matching this network spec.
+        """
+        # TODO(anyone): handle security_groups
+        net = self._validate_network(obj, net_spec, 'update')
+        selected_ports = []
+        for p in ports:
+            floating = p.get('floating', {})
+            floating_network = net.get(self.FLOATING_NETWORK, None)
+            if floating_network and floating.get(
+                    'floating_network_id') != floating_network:
+                continue
+            floating_ip_address = net.get(self.FLOATING_IP, None)
+            if floating_ip_address and floating.get(
+                    'floating_ip_address') != floating_ip_address:
+                continue
+            # If network properties didn't contain floating ip,
+            # then we should better not make a port with floating ip
+            # as candidate.
+            if (floating and not floating_network and not floating_ip_address):
+                continue
+            port_id = net.get(self.PORT, None)
+            if port_id and p['id'] != port_id:
+                continue
+            fixed_ip = net.get(self.FIXED_IP, None)
+            if fixed_ip:
+                fixed_ips = [ff['ip_address'] for ff in p['fixed_ips']]
+                if fixed_ip not in fixed_ips:
+                    continue
+            network = net.get(self.NETWORK, None)
+            if network:
+                net_id = self.network(obj).network_get(network).id
+                if p['network_id'] != net_id:
+                    continue
+            selected_ports.append(p)
+        return selected_ports
+
+    def _update_network_remove_port(self, obj, networks):
         """Delete existing interfaces from the node.
 
         :param obj: The node object to operate.
@@ -843,59 +1142,31 @@ class ServerProfile(base.Profile):
         :returns: ``None``
         :raises: ``EResourceUpdate``
         """
-        def _get_network(nc, net_id, server_id):
-            try:
-                net = nc.network_get(net_id)
-                return net.id
-            except exc.InternalError as ex:
-                raise exc.EResourceUpdate(type='server', id=server_id,
-                                          message=six.text_type(ex))
-
-        def _do_delete(port_id, server_id):
-            try:
-                cc.server_interface_delete(port_id, server_id)
-            except exc.InternalError as ex:
-                raise exc.EResourceUpdate(type='server', id=server_id,
-                                          message=six.text_type(ex))
-
         cc = self.compute(obj)
         nc = self.network(obj)
-        try:
-            existing = list(cc.server_interface_list(obj.physical_id))
-        except exc.InternalError as ex:
-            raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
-
-        ports = []
-        for intf in existing:
-            fixed_ips = [addr['ip_address'] for addr in intf.fixed_ips]
-            ports.append({
-                'id': intf.port_id,
-                'net': intf.net_id,
-                'ips': fixed_ips
-            })
+        internal_ports = obj.data.get('internal_ports', [])
 
         for n in networks:
-            network = n.get('network', None)
-            port = n.get('port', None)
-            fixed_ip = n.get('fixed_ip', None)
-            if port:
-                for p in ports:
-                    if p['id'] == port:
-                        ports.remove(p)
-                        _do_delete(port, obj.physical_id)
-            elif fixed_ip:
-                net_id = _get_network(nc, network, obj.physical_id)
-                for p in ports:
-                    if (fixed_ip in p['ips'] and net_id == p['net']):
-                        ports.remove(p)
-                        _do_delete(p['id'], obj.physical_id)
-            elif port is None and fixed_ip is None:
-                net_id = _get_network(nc, network, obj.physical_id)
-                for p in ports:
-                    if p['net'] == net_id:
-                        ports.remove(p)
-                        _do_delete(p['id'], obj.physical_id)
+            candidate_ports = self._find_port_by_net_spec(
+                obj, n, internal_ports)
+            port = candidate_ports[0]
+            try:
+                # Detach port from server
+                cc.server_interface_delete(port['id'], obj.physical_id)
+                # delete port if created by senlin
+                if port.get('remove', False):
+                    nc.port_delete(port['id'], ignore_missing=True)
+                # delete floating IP if created by senlin
+                if (port.get('floating', None) and
+                        port['floating'].get('remove', False)):
+                    nc.floatingip_delete(port['floating']['id'],
+                                         ignore_missing=True)
+            except exc.InternalError as ex:
+                raise exc.EResourceUpdate(type='server', id=obj.physical_id,
+                                          message=six.text_type(ex))
+            internal_ports.remove(port)
+        obj.data['internal_ports'] = internal_ports
+        node_obj.Node.update(self.context, obj.id, {'data': obj.data})
 
     def _update_network(self, obj, new_profile):
         """Updating server network interfaces.
@@ -916,11 +1187,11 @@ class ServerProfile(base.Profile):
 
         # Detach some existing interfaces
         if networks_delete:
-            self._delete_interfaces(obj, networks_delete)
+            self._update_network_remove_port(obj, networks_delete)
 
         # Attach new interfaces
         if networks_create:
-            self._create_interfaces(obj, networks_create)
+            self._update_network_add_port(obj, networks_create)
         return
 
     def do_update(self, obj, new_profile=None, **params):
@@ -1035,6 +1306,10 @@ class ServerProfile(base.Profile):
         if 'security_groups' in server_data:
             for sg in server_data['security_groups']:
                 sgroups.append(sg['name'])
+        # when we have multiple nics the info will include the
+        # security groups N times where N == number of nics. Be nice
+        # and only display it once.
+        sgroups = list(set(sgroups))
         if len(sgroups) == 0:
             details['security_groups'] = ''
         elif len(sgroups) == 1:
@@ -1044,6 +1319,85 @@ class ServerProfile(base.Profile):
 
         return dict((k, details[k]) for k in sorted(details))
 
+    def do_adopt(self, obj, overrides=None, snapshot=False):
+        """Adopt an existing server node for management.
+
+        :param obj: A node object for this operation. It could be a puppet
+            node that provides only 'user', 'project' and 'physical_id'
+            properties when doing a preview. It can be a real Node object for
+            node adoption.
+        :param overrides: A dict containing the properties that will be
+            overridden when generating a profile for the server.
+        :param snapshot: A boolean flag indicating whether the profile should
+            attempt a snapshot operation before adopting the server. If set to
+            True, the ID of the snapshot will be used as the image ID.
+
+        :returns: A dict containing the spec created from the server object or
+            a dict containing error information if failure occurred.
+        """
+        driver = self.compute(obj)
+
+        # TODO(Qiming): Add snapshot support
+        # snapshot = driver.snapshot_create(...)
+
+        error = {}
+        try:
+            server = driver.server_get(obj.physical_id)
+        except exc.InternalError as ex:
+            error = {'code': ex.code, 'message': six.text_type(ex)}
+
+        if error:
+            return {'Error': error}
+
+        spec = {}
+        # Context?
+        # TODO(Qiming): Need to fetch admin password from a different API
+        spec[self.AUTO_DISK_CONFIG] = server.disk_config == 'AUTO'
+
+        spec[self.AVAILABILITY_ZONE] = server.availability_zone
+
+        # TODO(Anyone): verify if this needs a format conversion
+        bdm = server.block_device_mapping or []
+        spec[self.BLOCK_DEVICE_MAPPING_V2] = bdm
+
+        spec[self.CONFIG_DRIVE] = server.has_config_drive or False
+        spec[self.FLAVOR] = server.flavor['id']
+        spec[self.IMAGE] = server.image['id'] or server.image
+        spec[self.KEY_NAME] = server.key_name
+
+        # metadata
+        metadata = server.metadata or {}
+        metadata.pop('cluster_id', None)
+        metadata.pop('cluster_node_id', None)
+        metadata.pop('cluster_node_index', None)
+        spec[self.METADATA] = metadata
+
+        # name
+        spec[self.NAME] = server.name
+
+        networks = server.addresses
+        net_list = []
+        for network, interfaces in networks.items():
+            for intf in interfaces:
+                ip_type = intf.get('OS-EXT-IPS:type')
+                if ip_type == 'fixed':
+                    net_list.append({self.NETWORK: network})
+
+        spec[self.NETWORKS] = net_list
+        # NOTE: the personality attribute is missing for ever.
+        spec[self.SECURITY_GROUPS] = [
+            sg['name'] for sg in server.security_groups
+        ]
+        # TODO(Qiming): get server user_data and parse it.
+        # Note: user_data is returned in 2.3 microversion API, in a different
+        # property name.
+        # spec[self.USER_DATA] = server.user_data
+
+        if overrides:
+            spec.update(overrides)
+
+        return spec
+
     def do_join(self, obj, cluster_id):
         if not obj.physical_id:
             return False
@@ -1080,15 +1434,28 @@ class ServerProfile(base.Profile):
         return True
 
     def do_recover(self, obj, **options):
-        # NOTE: We do a 'get' not a 'pop' here, because the operations may
-        #       get fall back to the base class for handling
+        """Handler for recover operation.
+
+        :param obj: The node object.
+        :param dict options: A list for operations each of which has a name
+            and optionally a map from parameter to values.
+        """
         operation = options.get('operation', None)
 
         if operation and not isinstance(operation, six.string_types):
             operation = operation[0]
 
-        if operation == 'REBUILD':
-            return self.handle_rebuild(obj)
+        if operation is not None and 'name' in operation:
+            op_name = operation['name']
+            if op_name.upper() != consts.RECOVER_RECREATE:
+                op_params = operation.get('params', {})
+                if op_name.lower() not in self.OP_NAMES:
+                    LOG.error("The operation '%s' is not supported",
+                              op_name)
+                    return False
+
+                method = getattr(self, "handle_" + op_name.lower())
+                return method(obj, **op_params)
 
         return super(ServerProfile, self).do_recover(obj, **options)
 
