@@ -16,6 +16,8 @@ from tempest import config
 
 from senlin.tests.tempest import base
 from senlin.tests.tempest.common import clustering_client
+from senlin.tests.tempest.common import compute_client
+from senlin.tests.tempest.common import network_client
 
 CONF = config.CONF
 
@@ -38,3 +40,17 @@ class BaseSenlinAPITest(base.BaseSenlinTest):
             CONF.identity.region,
             **cls.default_params_with_timeout_values
         )
+
+        cls.compute_client = compute_client.V21ComputeClient(
+            cls.os.auth_provider,
+            CONF.compute.catalog_type,
+            CONF.identity.region,
+            **cls.default_params_with_timeout_values
+        )
+
+        cls.network_client = network_client.NetworkClient(
+            cls.os.auth_provider,
+            CONF.network.catalog_type,
+            CONF.identity.region,
+            **cls.default_params_with_timeout_values
+        )
