@@ -12,14 +12,13 @@
 
 from tempest.lib import decorators
 from tempest.lib import exceptions
-from tempest import test
 
 from senlin.tests.tempest.api import base
 
 
 class TestProfileTypeShowNegativeNotFound(base.BaseSenlinAPITest):
 
-    @test.attr(type=['negative'])
+    @decorators.attr(type=['negative'])
     @decorators.idempotent_id('58181c56-3166-4478-8981-e1d476065f2b')
     def test_profile_type_show_not_found(self):
         ex = self.assertRaises(exceptions.NotFound,
