@@ -10,10 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import json
-import uuid
 
+from oslo_serialization import jsonutils
 from oslo_utils import timeutils as tu
+from oslo_utils import uuidutils
 
 from senlin.db.sqlalchemy import api as db_api
 from senlin.engine import parser
@@ -29,7 +29,6 @@ sample_profile = '''
         myrandom: OS::Heat::RandomString
       files:
         myfile: contents
-  permission: xxxyyy
 '''
 
 sample_action = '''
@@ -48,7 +47,7 @@ sample_action = '''
 '''
 
 
-UUIDs = (UUID1, UUID2, UUID3) = sorted([str(uuid.uuid4())
+UUIDs = (UUID1, UUID2, UUID3) = sorted([uuidutils.generate_uuid()
                                         for x in range(3)])
 
 
@@ -72,10 +71,12 @@ def create_cluster(ctx, profile, **kwargs):
         'next_index': 1,
         'timeout': 60,
         'desired_capacity': 0,
-        'init_time': tu.utcnow(),
+        'init_at': tu.utcnow(True),
         'status': 'INIT',
         'status_reason': 'Just Initialized',
-        'metadata': {},
+        'meta_data': {},
+        'dependents': {},
+        'config': {},
     }
     values.update(kwargs)
     if 'project' in kwargs:
@@ -88,7 +89,7 @@ def create_node(ctx, cluster, profile, **kwargs):
         cluster_id = cluster.id
         index = db_api.cluster_next_index(ctx, cluster_id)
     else:
-        cluster_id = None
+        cluster_id = ''
         index = -1
 
     values = {
@@ -99,13 +100,13 @@ def create_node(ctx, cluster, profile, **kwargs):
         'project': ctx.project,
         'index': index,
         'role': None,
-        'created_time': None,
-        'updated_time': None,
-        'deleted_time': None,
+        'created_at': None,
+        'updated_at': None,
         'status': 'ACTIVE',
         'status_reason': 'create complete',
-        'metadata': json.loads('{"foo": "123"}'),
-        'data': json.loads('{"key1": "value1"}'),
+        'meta_data': jsonutils.loads('{"foo": "123"}'),
+        'data': jsonutils.loads('{"key1": "value1"}'),
+        'dependents': {},
     }
     values.update(kwargs)
     return db_api.node_create(ctx, values)
@@ -117,8 +118,7 @@ def create_webhook(ctx, obj_id, obj_type, action, **kwargs):
         'user': ctx.user,
         'project': ctx.project,
         'domain': ctx.domain,
-        'created_time': None,
-        'deleted_time': None,
+        'created_at': None,
         'obj_id': obj_id,
         'obj_type': obj_type,
         'action': action,
@@ -145,3 +145,47 @@ def create_action(ctx, **kwargs):
     }
     values.update(kwargs)
     return db_api.action_create(ctx, values)
+
+
+def create_policy(ctx, **kwargs):
+    values = {
+        'name': 'test_policy',
+        'type': 'senlin.policy.scaling',
+        'user': ctx.user,
+        'project': ctx.project,
+        'domain': ctx.domain,
+        'spec': {
+            'type': 'senlin.policy.scaling',
+            'version': '1.0',
+            'properties': {
+                'adjustment_type': 'WHATEVER',
+                'count': 1,
+            }
+        },
+        'data': None,
+    }
+
+    values.update(kwargs)
+    return db_api.policy_create(ctx, values)
+
+
+def create_event(ctx, **kwargs):
+    values = {
+        'timestamp': tu.utcnow(True),
+        'obj_id': 'FAKE_ID',
+        'obj_name': 'FAKE_NAME',
+        'obj_type': 'CLUSTER',
+        'cluster_id': 'FAKE_CLUSTER',
+        'level': '20',
+        'user': ctx.user,
+        'project': ctx.project,
+        'action': 'DANCE',
+        'status': 'READY',
+        'status_reason': 'Just created.',
+        'meta_data': {
+            'air': 'polluted'
+        }
+    }
+
+    values.update(kwargs)
+    return db_api.event_create(ctx, values)
