@@ -10,10 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import logging
-
+from oslo_log import log as logging
+from oslo_utils import reflection
 from oslo_utils import timeutils as tu
 
+from senlin.common import utils as common_utils
 from senlin.db.sqlalchemy import api as db_api
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
@@ -33,13 +34,14 @@ class DBAPIEventTest(base.SenlinTestCase):
 
     def create_event(self, ctx, timestamp=None, level=logging.INFO,
                      entity=None, action=None, status=None,
-                     status_reason=None, deleted_time=None):
+                     status_reason=None):
 
         fake_timestamp = tu.parse_strtime(
             '2014-12-19 11:51:54.670244', '%Y-%m-%d %H:%M:%S.%f')
 
         if entity:
-            type_name = entity.__class__.__name__.upper()
+            e_name = reflection.get_class_name(entity, fully_qualified=False)
+            type_name = e_name.upper()
             if type_name == 'CLUSTER':
                 cluster_id = entity.id
             elif type_name == 'NODE':
@@ -53,16 +55,15 @@ class DBAPIEventTest(base.SenlinTestCase):
         values = {
             'timestamp': timestamp or fake_timestamp,
             'level': level,
-            'obj_id': entity.id if entity else '',
-            'obj_name': entity.name if entity else '',
-            'obj_type': type_name,
+            'oid': entity.id if entity else '',
+            'oname': entity.name if entity else '',
+            'otype': type_name,
             'cluster_id': cluster_id,
             'action': action or '',
             'status': status or '',
             'status_reason': status_reason or '',
             'user': ctx.user,
             'project': ctx.project,
-            'deleted_time': deleted_time,
         }
 
         # Make sure all fields can be customized
@@ -74,17 +75,37 @@ class DBAPIEventTest(base.SenlinTestCase):
         self.assertIsNotNone(ret_event)
         tst_timestamp = tu.parse_strtime('2014-12-19 11:51:54.670244',
                                          '%Y-%m-%d %H:%M:%S.%f')
-        self.assertEqual(tst_timestamp, ret_event.timestamp)
-        self.assertEqual(logging.INFO, ret_event.level)
-        self.assertEqual('', ret_event.obj_id)
-        self.assertEqual('', ret_event.obj_type)
-        self.assertEqual('', ret_event.obj_name)
+
+        self.assertEqual(common_utils.isotime(tst_timestamp),
+                         common_utils.isotime(ret_event.timestamp))
+        self.assertEqual('20', ret_event.level)
+        self.assertEqual('', ret_event.oid)
+        self.assertEqual('', ret_event.otype)
+        self.assertEqual('', ret_event.oname)
         self.assertEqual('', ret_event.action)
         self.assertEqual('', ret_event.status)
         self.assertEqual('', ret_event.status_reason)
         self.assertEqual(self.ctx.user, ret_event.user)
         self.assertEqual(self.ctx.project, ret_event.project)
 
+    def test_event_get_diff_project(self):
+        event = self.create_event(self.ctx)
+        new_ctx = utils.dummy_context(project='a-different-project')
+        res = db_api.event_get(new_ctx, event.id)
+        self.assertIsNone(res)
+        res = db_api.event_get(new_ctx, event.id, project_safe=False)
+        self.assertIsNotNone(res)
+        self.assertEqual(event.id, res.id)
+
+    def test_event_get_admin_context(self):
+        event = self.create_event(self.ctx)
+        admin_ctx = utils.dummy_context(project='a-different-project',
+                                        is_admin=True)
+        res = db_api.event_get(admin_ctx, event.id, project_safe=True)
+        self.assertIsNone(res)
+        res = db_api.event_get(admin_ctx, event.id, project_safe=False)
+        self.assertIsNotNone(res)
+
     def test_event_get_by_short_id(self):
         event = self.create_event(self.ctx)
         short_id = event.id[:6]
@@ -98,6 +119,18 @@ class DBAPIEventTest(base.SenlinTestCase):
         ret_event = db_api.event_get_by_short_id(self.ctx, 'non-existent')
         self.assertIsNone(ret_event)
 
+    def test_event_get_by_short_id_diff_project(self):
+        event = self.create_event(self.ctx)
+
+        new_ctx = utils.dummy_context(project='a-different-project')
+        short_id = event.id[:8]
+        res = db_api.event_get_by_short_id(new_ctx, short_id)
+        self.assertIsNone(res)
+        res = db_api.event_get_by_short_id(new_ctx, short_id,
+                                           project_safe=False)
+        self.assertIsNotNone(res)
+        self.assertEqual(event.id, res.id)
+
     def test_event_get_all(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile)
         cluster2 = shared.create_cluster(self.ctx, self.profile)
@@ -110,17 +143,13 @@ class DBAPIEventTest(base.SenlinTestCase):
         events = db_api.event_get_all(self.ctx)
         self.assertEqual(3, len(events))
 
-        cluster_ids = [event.obj_id for event in events]
-        obj_names = [event.obj_name for event in events]
+        cluster_ids = [event.oid for event in events]
+        onames = [event.oname for event in events]
 
         self.assertIn(cluster1.id, cluster_ids)
-        self.assertIn(cluster1.name, obj_names)
+        self.assertIn(cluster1.name, onames)
         self.assertIn(cluster2.id, cluster_ids)
-        self.assertIn(cluster2.name, obj_names)
-
-        # Set project_safe to false
-        events = db_api.event_get_all(self.ctx, project_safe=False)
-        self.assertEqual(3, len(events))
+        self.assertIn(cluster2.name, onames)
 
     def test_event_get_all_with_limit(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile)
@@ -165,58 +194,39 @@ class DBAPIEventTest(base.SenlinTestCase):
         self.assertEqual(1, len(events))
         self.assertEqual(event2_id, events[0].id)
 
-    def test_event_get_all_with_sort_keys_and_dir(self):
+    def test_event_get_all_with_sorting(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile)
 
         event1 = self.create_event(self.ctx, entity=cluster1,
-                                   timestamp=tu.utcnow(),
+                                   timestamp=tu.utcnow(True),
                                    action='action2')
         event2 = self.create_event(self.ctx, entity=cluster1,
-                                   timestamp=tu.utcnow(),
+                                   timestamp=tu.utcnow(True),
                                    action='action3')
         event3 = self.create_event(self.ctx, entity=cluster1,
-                                   timestamp=tu.utcnow(),
+                                   timestamp=tu.utcnow(True),
                                    action='action1')
 
-        events = db_api.event_get_all(self.ctx, sort_keys=['timestamp'])
+        events = db_api.event_get_all(self.ctx, sort='timestamp')
         self.assertEqual(event1.id, events[0].id)
         self.assertEqual(event2.id, events[1].id)
         self.assertEqual(event3.id, events[2].id)
 
-        events = db_api.event_get_all(self.ctx, sort_keys=['timestamp'],
-                                      sort_dir='desc')
+        events = db_api.event_get_all(self.ctx, sort='timestamp:desc')
         self.assertEqual(event1.id, events[2].id)
         self.assertEqual(event2.id, events[1].id)
         self.assertEqual(event3.id, events[0].id)
 
-        events = db_api.event_get_all(self.ctx, sort_keys=['action'])
+        events = db_api.event_get_all(self.ctx, sort='action')
         self.assertEqual(event1.id, events[1].id)
         self.assertEqual(event2.id, events[2].id)
         self.assertEqual(event3.id, events[0].id)
 
-        events = db_api.event_get_all(self.ctx, sort_keys=['action'],
-                                      sort_dir='desc')
+        events = db_api.event_get_all(self.ctx, sort='action:desc')
         self.assertEqual(event1.id, events[1].id)
         self.assertEqual(event2.id, events[0].id)
         self.assertEqual(event3.id, events[2].id)
 
-    def test_event_get_all_show_deleted(self):
-        cluster1 = shared.create_cluster(self.ctx, self.profile)
-        cluster2 = shared.create_cluster(self.ctx, self.profile)
-
-        # Simulate deleted events by setting 'deleted_time' to not-None
-        now = tu.utcnow()
-        self.create_event(self.ctx, entity=cluster1, deleted_time=now)
-        self.create_event(self.ctx, entity=cluster1)
-        self.create_event(self.ctx, entity=cluster2, deleted_time=now)
-
-        # Default show_deleted is False
-        events = db_api.event_get_all(self.ctx)
-        self.assertEqual(1, len(events))
-
-        events = db_api.event_get_all(self.ctx, show_deleted=True)
-        self.assertEqual(3, len(events))
-
     def test_event_get_all_project_safe(self):
         self.ctx.project = 'project_1'
         cluster1 = shared.create_cluster(self.ctx, self.profile,
@@ -232,23 +242,41 @@ class DBAPIEventTest(base.SenlinTestCase):
         events = db_api.event_get_all(self.ctx)
         self.assertEqual(2, len(events))
 
-        obj_ids = [event.obj_id for event in events]
-        obj_names = [event.obj_name for event in events]
-        self.assertNotIn(cluster1.id, obj_ids)
-        self.assertNotIn(cluster1.name, obj_names)
-        self.assertIn(cluster2.id, obj_ids)
-        self.assertIn(cluster2.name, obj_names)
+        oids = [event.oid for event in events]
+        onames = [event.oname for event in events]
+        self.assertNotIn(cluster1.id, oids)
+        self.assertNotIn(cluster1.name, onames)
+        self.assertIn(cluster2.id, oids)
+        self.assertIn(cluster2.name, onames)
 
         # Set project_safe to false, we should get all three events
         events = db_api.event_get_all(self.ctx, project_safe=False)
         self.assertEqual(3, len(events))
 
-        obj_ids = [event.obj_id for event in events]
-        obj_names = [event.obj_name for event in events]
-        self.assertIn(cluster1.id, obj_ids)
-        self.assertIn(cluster1.name, obj_names)
-        self.assertIn(cluster2.id, obj_ids)
-        self.assertIn(cluster2.name, obj_names)
+        oids = [event.oid for event in events]
+        onames = [event.oname for event in events]
+        self.assertIn(cluster1.id, oids)
+        self.assertIn(cluster1.name, onames)
+        self.assertIn(cluster2.id, oids)
+        self.assertIn(cluster2.name, onames)
+
+    def test_event_get_all_admin_context(self):
+        self.ctx.project = 'project_1'
+        cluster1 = shared.create_cluster(self.ctx, self.profile,
+                                         name='cluster1')
+        self.create_event(self.ctx, entity=cluster1)
+        self.ctx.project = 'project_2'
+        cluster2 = shared.create_cluster(self.ctx, self.profile,
+                                         name='cluster2')
+        self.create_event(self.ctx, entity=cluster2, action='CLUSTER_CREATE')
+        self.create_event(self.ctx, entity=cluster2, action='CLUSTER_DELETE')
+
+        admin_ctx = utils.dummy_context(project='another-project',
+                                        is_admin=True)
+        events = db_api.event_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(events))
+        events = db_api.event_get_all(admin_ctx, project_safe=False)
+        self.assertEqual(3, len(events))
 
     def test_event_get_all_by_cluster(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile)
@@ -304,6 +332,33 @@ class DBAPIEventTest(base.SenlinTestCase):
         events = db_api.event_get_all_by_cluster(self.ctx, cluster2.id)
         self.assertEqual(3, len(events))
 
+    def test_event_get_all_by_cluster_diff_project(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+        cluster2 = shared.create_cluster(self.ctx, self.profile)
+        self.create_event(self.ctx, entity=cluster1)
+        self.create_event(self.ctx, entity=cluster2)
+
+        new_ctx = utils.dummy_context(project='a-different-project')
+        events = db_api.event_get_all_by_cluster(new_ctx, cluster1.id)
+        self.assertEqual(0, len(events))
+        events = db_api.event_get_all_by_cluster(new_ctx, cluster1.id,
+                                                 project_safe=False)
+        self.assertEqual(1, len(events))
+
+    def test_event_get_all_by_cluster_admin_context(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+        self.create_event(self.ctx, entity=cluster1)
+        self.create_event(self.ctx, entity=cluster1)
+
+        admin_ctx = utils.dummy_context(project='a-different-project',
+                                        is_admin=True)
+        events = db_api.event_get_all_by_cluster(admin_ctx, cluster1.id,
+                                                 project_safe=True)
+        self.assertEqual(0, len(events))
+        events = db_api.event_get_all_by_cluster(admin_ctx, cluster1.id,
+                                                 project_safe=False)
+        self.assertEqual(2, len(events))
+
     def test_event_count_all_by_cluster(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile)
         cluster2 = shared.create_cluster(self.ctx, self.profile)
@@ -346,6 +401,33 @@ class DBAPIEventTest(base.SenlinTestCase):
         self.assertEqual(1, db_api.event_count_by_cluster(self.ctx,
                                                           cluster2.id))
 
+    def test_event_count_all_by_cluster_diff_project(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+        cluster2 = shared.create_cluster(self.ctx, self.profile)
+        self.create_event(self.ctx, entity=cluster1)
+        self.create_event(self.ctx, entity=cluster2)
+
+        new_ctx = utils.dummy_context(project='a-different-project')
+        res = db_api.event_count_by_cluster(new_ctx, cluster1.id)
+        self.assertEqual(0, res)
+        res = db_api.event_count_by_cluster(new_ctx, cluster1.id,
+                                            project_safe=False)
+        self.assertEqual(1, res)
+
+    def test_event_count_all_by_cluster_admin_context(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+        self.create_event(self.ctx, entity=cluster1)
+
+        admin_ctx = utils.dummy_context(project='a-different-project',
+                                        is_admin=True)
+
+        res = db_api.event_count_by_cluster(admin_ctx, cluster1.id,
+                                            project_safe=True)
+        self.assertEqual(0, res)
+        res = db_api.event_count_by_cluster(admin_ctx, cluster1.id,
+                                            project_safe=False)
+        self.assertEqual(1, res)
+
     def test_event_get_all_filtered(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile,
                                          name='cluster1')
@@ -371,27 +453,27 @@ class DBAPIEventTest(base.SenlinTestCase):
                                                  filters=filters)
         self.assertEqual(0, len(events))
 
-        # test filter by obj_name
-        filters = {'obj_name': 'cluster1'}
+        # test filter by oname
+        filters = {'oname': 'cluster1'}
         events = db_api.event_get_all_by_cluster(self.ctx, cluster1.id,
                                                  filters=filters)
         self.assertEqual(2, len(events))
-        self.assertEqual('cluster1', events[0].obj_name)
-        self.assertEqual('cluster1', events[1].obj_name)
+        self.assertEqual('cluster1', events[0].oname)
+        self.assertEqual('cluster1', events[1].oname)
 
-        filters = {'obj_name': 'cluster3'}
+        filters = {'oname': 'cluster3'}
         events = db_api.event_get_all_by_cluster(self.ctx, cluster1.id,
                                                  filters=filters)
         self.assertEqual(0, len(events))
 
-        # test filter by obj_type
-        filters = {'obj_type': 'CLUSTER'}
+        # test filter by otype
+        filters = {'otype': 'CLUSTER'}
         events = db_api.event_get_all_by_cluster(self.ctx, cluster2.id,
                                                  filters=filters)
         self.assertEqual(1, len(events))
-        self.assertEqual('CLUSTER', events[0].obj_type)
+        self.assertEqual('CLUSTER', events[0].otype)
 
-        filters = {'obj_type': 'NODE'}
+        filters = {'otype': 'NODE'}
         events = db_api.event_get_all_by_cluster(self.ctx, cluster2.id,
                                                  filters=filters)
         self.assertEqual(0, len(events))
@@ -404,3 +486,65 @@ class DBAPIEventTest(base.SenlinTestCase):
                                                  limit=1, marker=marker)
         self.assertEqual(1, len(events))
         self.assertEqual(expected, events[0].id)
+
+    def test_event_prune(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+        cluster2 = shared.create_cluster(self.ctx, self.profile)
+        node1_1 = shared.create_node(self.ctx, cluster1, self.profile)
+        node_orphan = shared.create_node(self.ctx, None, self.profile)
+
+        # prune 1: cluster events
+        self.create_event(self.ctx, entity=cluster1)
+        self.create_event(self.ctx, entity=cluster1)
+
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(2, len(res))
+        db_api.event_prune(self.ctx, cluster1.id)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(0, len(res))
+
+        # prune 2: Node level events account to cluster
+        self.create_event(self.ctx, entity=node1_1)
+
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(1, len(res))
+        db_api.event_prune(self.ctx, cluster1.id)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(0, len(res))
+
+        # prune 3: Events related to orphan nodes
+        # no impact here and no error given
+        self.create_event(self.ctx, entity=node_orphan)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(0, len(res))
+        db_api.event_prune(self.ctx, cluster1.id)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(0, len(res))
+
+        # prune 4: Another cluster
+        # no impact here and no error given
+        self.create_event(self.ctx, entity=cluster2)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(0, len(res))
+        db_api.event_prune(self.ctx, cluster1.id)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(0, len(res))
+
+    def test_event_purge(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+        node1_1 = shared.create_node(self.ctx, cluster1, self.profile)
+        node1_2 = shared.create_node(self.ctx, cluster1, self.profile)
+
+        self.create_event(self.ctx, entity=cluster1, status='start')
+        self.create_event(self.ctx, entity=cluster1, status='end')
+        self.create_event(self.ctx, entity=node1_1, status='start')
+        self.create_event(self.ctx, entity=node1_1, status='end')
+        timestamp = tu.utcnow()
+        self.create_event(self.ctx, timestamp=timestamp,
+                          entity=node1_2, status='start')
+
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(5, len(res))
+        db_api.event_purge(project=None, granularity='days', age=5)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(1, len(res))
