@@ -41,16 +41,35 @@ class TestNeutronV2Driver(base.SenlinTestCase):
 
         self.conn.network.find_network.return_value = network_obj
         res = self.nc.network_get(net_id)
-        self.conn.network.find_network.assert_called_once_with(net_id)
+        self.conn.network.find_network.assert_called_once_with(net_id, False)
         self.assertEqual(network_obj, res)
 
+    def test_port_find(self):
+        port_id = 'port_identifier'
+        port_obj = mock.Mock()
+
+        self.conn.network.find_port.return_value = port_obj
+        res = self.nc.port_find(port_id)
+        self.conn.network.find_port.assert_called_once_with(port_id, False)
+        self.assertEqual(port_obj, res)
+
+    def test_security_group_find(self):
+        sg_id = 'sg_identifier'
+        sg_obj = mock.Mock()
+
+        self.conn.network.find_security_group.return_value = sg_obj
+        res = self.nc.security_group_find(sg_id)
+        self.conn.network.find_security_group.assert_called_once_with(
+            sg_id, False)
+        self.assertEqual(sg_obj, res)
+
     def test_subnet_get(self):
         subnet_id = 'subnet_identifier'
         subnet_obj = mock.Mock()
 
         self.conn.network.find_subnet.return_value = subnet_obj
         res = self.nc.subnet_get(subnet_id)
-        self.conn.network.find_subnet.assert_called_once_with(subnet_id)
+        self.conn.network.find_subnet.assert_called_once_with(subnet_id, False)
         self.assertEqual(subnet_obj, res)
 
     def test_loadbalancer_get(self):
@@ -59,16 +78,10 @@ class TestNeutronV2Driver(base.SenlinTestCase):
 
         self.conn.network.find_load_balancer.return_value = loadbalancer_obj
         res = self.nc.loadbalancer_get(lb_id)
-        self.conn.network.find_load_balancer.assert_called_once_with(lb_id)
+        self.conn.network.find_load_balancer.assert_called_once_with(lb_id,
+                                                                     False)
         self.assertEqual(loadbalancer_obj, res)
 
-    def test_loadbalancer_list(self):
-        loadbalancers = ['lb1', 'lb2']
-
-        self.conn.network.load_balancers.return_value = loadbalancers
-        self.assertEqual(loadbalancers, self.nc.loadbalancer_list())
-        self.conn.network.load_balancers.assert_called_once_with()
-
     def test_loadbalancer_create(self):
         vip_subnet_id = 'ID1'
         lb_obj = mock.Mock()
@@ -106,23 +119,6 @@ class TestNeutronV2Driver(base.SenlinTestCase):
         self.conn.network.delete_load_balancer.assert_called_with(
             lb_id, ignore_missing=True)
 
-    def test_listener_get(self):
-        name_or_id = 'listener_identifier'
-        listener_obj = mock.Mock()
-
-        self.conn.network.find_listener.return_value = listener_obj
-        res = self.nc.listener_get(name_or_id)
-        self.conn.network.find_listener.assert_called_once_with(
-            name_or_id)
-        self.assertEqual(listener_obj, res)
-
-    def test_listener_list(self):
-        listeners = ['listener1', 'listener2']
-
-        self.conn.network.listeners.return_value = listeners
-        self.assertEqual(listeners, self.nc.listener_list())
-        self.conn.network.listeners.assert_called_once_with()
-
     def test_listener_create(self):
         loadbalancer_id = 'ID1'
         protocol = 'HTTP'
@@ -165,22 +161,6 @@ class TestNeutronV2Driver(base.SenlinTestCase):
         self.conn.network.delete_listener.assert_called_with(
             listener_id, ignore_missing=True)
 
-    def test_pool_get(self):
-        name_or_id = 'pool_identifier'
-        pool_obj = mock.Mock()
-
-        self.conn.network.find_pool.return_value = pool_obj
-        res = self.nc.pool_get(name_or_id)
-        self.conn.network.find_pool.assert_called_once_with(name_or_id)
-        self.assertEqual(pool_obj, res)
-
-    def test_pool_list(self):
-        pools = ['pool1', 'pool2']
-
-        self.conn.network.pools.return_value = pools
-        self.assertEqual(pools, self.nc.pool_list())
-        self.conn.network.pools.assert_called_once_with()
-
     def test_pool_create(self):
         lb_algorithm = 'ROUND_ROBIN'
         listener_id = 'ID1'
@@ -222,25 +202,6 @@ class TestNeutronV2Driver(base.SenlinTestCase):
         self.conn.network.delete_pool.assert_called_with(
             pool_id, ignore_missing=True)
 
-    def test_pool_member_get(self):
-        name_or_id = 'member_identifier'
-        pool_id = 'ID1'
-        member_obj = mock.Mock()
-
-        self.conn.network.find_pool_member.return_value = member_obj
-        res = self.nc.pool_member_get(pool_id, name_or_id)
-        self.conn.network.find_pool_member.assert_called_once_with(
-            name_or_id, pool_id)
-        self.assertEqual(member_obj, res)
-
-    def test_pool_member_list(self):
-        pool_id = 'ID1'
-        members = ['member1', 'member2']
-
-        self.conn.network.pool_members.return_value = members
-        self.assertEqual(members, self.nc.pool_member_list(pool_id))
-        self.conn.network.pool_members.assert_called_once_with(pool_id)
-
     def test_pool_member_create(self):
         pool_id = 'ID1'
         address = '192.168.1.100'
@@ -259,7 +220,7 @@ class TestNeutronV2Driver(base.SenlinTestCase):
         self.assertEqual(member_obj, self.nc.pool_member_create(
             pool_id, address, protocol_port, subnet_id, **kwargs))
         self.conn.network.create_pool_member.assert_called_once_with(
-            pool_id=pool_id, address=address, protocol_port=protocol_port,
+            pool_id, address=address, protocol_port=protocol_port,
             subnet_id=subnet_id, **kwargs)
 
         # Use default input parameters
@@ -269,7 +230,7 @@ class TestNeutronV2Driver(base.SenlinTestCase):
         self.assertEqual(member_obj, self.nc.pool_member_create(
             pool_id, address, protocol_port, subnet_id))
         self.conn.network.create_pool_member.assert_called_with(
-            pool_id=pool_id, address=address, protocol_port=protocol_port,
+            pool_id, address=address, protocol_port=protocol_port,
             subnet_id=subnet_id, **kwargs)
 
     def test_pool_member_delete(self):
@@ -284,23 +245,6 @@ class TestNeutronV2Driver(base.SenlinTestCase):
         self.conn.network.delete_pool_member.assert_called_with(
             member_id, pool_id, ignore_missing=True)
 
-    def test_healthmonitor_get(self):
-        name_or_id = 'healthmonitor_identifier'
-        healthmonitor_obj = mock.Mock()
-
-        self.conn.network.find_health_monitor.return_value = healthmonitor_obj
-        res = self.nc.healthmonitor_get(name_or_id)
-        self.conn.network.find_health_monitor.assert_called_once_with(
-            name_or_id)
-        self.assertEqual(healthmonitor_obj, res)
-
-    def test_healthmonitor_list(self):
-        healthmonitors = ['hm1', 'hm2']
-
-        self.conn.network.health_monitors.return_value = healthmonitors
-        self.assertEqual(healthmonitors, self.nc.healthmonitor_list())
-        self.conn.network.health_monitors.assert_called_once_with()
-
     def test_healthmonitor_create(self):
         hm_type = 'HTTP'
         delay = 30
@@ -354,3 +298,64 @@ class TestNeutronV2Driver(base.SenlinTestCase):
         self.nc.healthmonitor_delete(healthmonitor_id)
         self.conn.network.delete_health_monitor.assert_called_with(
             healthmonitor_id, ignore_missing=True)
+
+    def test_port_create(self):
+        port_attr = {
+            'network_id': 'foo'
+        }
+        self.nc.port_create(**port_attr)
+        self.conn.network.create_port.assert_called_once_with(
+            network_id='foo')
+
+    def test_port_delete(self):
+        self.nc.port_delete(port='foo')
+        self.conn.network.delete_port.assert_called_once_with(
+            port='foo', ignore_missing=True)
+
+    def test_port_update(self):
+        attr = {
+            'name': 'new_name'
+        }
+        self.nc.port_update('fake_port', **attr)
+        self.conn.network.update_port.assert_called_once_with(
+            'fake_port', **attr)
+
+    def test_floatingip_find(self):
+        floatingip_id = 'fake_id'
+        fip_obj = mock.Mock()
+
+        self.conn.network.find_ip.return_value = fip_obj
+        res = self.nc.floatingip_find(floatingip_id)
+        self.conn.network.find_ip.assert_called_once_with(
+            floatingip_id, ignore_missing=False)
+        self.assertEqual(fip_obj, res)
+
+    def test_floatingip_list_by_port_id(self):
+        port_id = 'port_id'
+        fip_obj_iter = iter([mock.Mock()])
+
+        self.conn.network.ips.return_value = fip_obj_iter
+        res = self.nc.floatingip_list(port=port_id)
+        self.conn.network.ips.assert_called_once_with(port_id=port_id)
+        self.assertEqual(1, len(res))
+
+    def test_floatingip_create(self):
+        attr = {
+            'network_id': 'foo'
+        }
+        self.nc.floatingip_create(**attr)
+        self.conn.network.create_ip.assert_called_once_with(
+            network_id='foo')
+
+    def test_floatingip_delete(self):
+        self.nc.floatingip_delete(floating_ip='foo')
+        self.conn.network.delete_ip.assert_called_once_with(
+            'foo', ignore_missing=True)
+
+    def test_floatingip_update(self):
+        attr = {
+            'port_id': 'fake_port'
+        }
+        self.nc.floatingip_update('fake_floatingip', **attr)
+        self.conn.network.update_ip.assert_called_once_with(
+            'fake_floatingip', **attr)
