@@ -63,6 +63,24 @@ class TestNovaV2(base.SenlinTestCase):
         d.image_find('foo', False)
         self.compute.find_image.assert_called_once_with('foo', False)
 
+    def test_keypair_create(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        d.keypair_create(name='foo')
+        self.compute.create_keypair.assert_called_once_with(name='foo')
+
+    def test_keypair_delete(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        d.keypair_delete('foo')
+        self.compute.delete_keypair.assert_called_once_with('foo', False)
+        self.compute.delete_keypair.reset_mock()
+
+        d.keypair_delete('foo', True)
+        self.compute.delete_keypair.assert_called_once_with('foo', True)
+        self.compute.delete_keypair.reset_mock()
+
+        d.keypair_delete('foo', False)
+        self.compute.delete_keypair.assert_called_once_with('foo', False)
+
     def test_keypair_find(self):
         d = nova_v2.NovaClient(self.conn_params)
         d.keypair_find('foo')
@@ -337,7 +355,7 @@ class TestNovaV2(base.SenlinTestCase):
         self.compute.wait_for_delete.assert_called_once_with('FOO', wait=120)
 
     def test_wait_for_server_delete_with_default_timeout(self):
-        cfg.CONF.set_override('default_action_timeout', 360, enforce_type=True)
+        cfg.CONF.set_override('default_action_timeout', 360)
         self.compute.find_server.return_value = 'FOO'
 
         d = nova_v2.NovaClient(self.conn_params)
@@ -387,6 +405,22 @@ class TestNovaV2(base.SenlinTestCase):
         self.compute.delete_server_interface.assert_called_once_with(
             'foo', server, True)
 
+    def test_server_floatingip_associate(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        d.server_floatingip_associate(server, 'fake_floatingip')
+        self.compute.add_floating_ip_to_server.assert_called_once_with(
+            server, 'fake_floatingip'
+        )
+
+    def test_server_floatingip_disassociate(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        d.server_floatingip_disassociate(server, 'fake_floatingip')
+        self.compute.remove_floating_ip_from_server.assert_called_once_with(
+            server, 'fake_floatingip'
+        )
+
     def test_server_metadata_get(self):
         server = mock.Mock()
         res_server = mock.Mock()
@@ -505,3 +539,17 @@ class TestNovaV2(base.SenlinTestCase):
         d.service_force_down(service)
         self.compute.force_service_down.assert_called_once_with(
             service, service.host, service.binary)
+
+    def test_create_volume_attachment(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        kwargs = {
+            'serverId': server,
+            'volumeId': 'fake_volume',
+        }
+        d.create_volume_attachment(server, **kwargs)
+
+    def test_delete_volume_attachment(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        d.delete_volume_attachment('fake_volume', server, ignore_missing=True)
