@@ -47,8 +47,7 @@ class ClusterTest(base.SenlinTestCase):
     @mock.patch.object(co.Cluster, 'count_all')
     def test_check_cluster_quota(self, mock_count):
         mock_count.return_value = 10
-        cfg.CONF.set_override('max_clusters_per_project', 11,
-                              enforce_type=True)
+        cfg.CONF.set_override('max_clusters_per_project', 11)
 
         res = self.eng.check_cluster_quota(self.ctx)
 
@@ -58,8 +57,7 @@ class ClusterTest(base.SenlinTestCase):
     @mock.patch.object(co.Cluster, 'count_all')
     def test_check_cluster_quota_failed(self, mock_count):
         mock_count.return_value = 11
-        cfg.CONF.set_override('max_clusters_per_project', 11,
-                              enforce_type=True)
+        cfg.CONF.set_override('max_clusters_per_project', 11)
 
         ex = self.assertRaises(exc.Forbidden,
                                self.eng.check_cluster_quota, self.ctx)
@@ -145,6 +143,51 @@ class ClusterTest(base.SenlinTestCase):
             dict(name='C1', desired_capacity=3, profile_id='PROFILE_ID',
                  min_size=0, max_size=-1, timeout=3600, metadata={},
                  dependents={}, data={}, next_index=1, status='INIT',
+                 config={},
+                 status_reason='Initializing', user=self.ctx.user,
+                 project=self.ctx.project, domain=self.ctx.domain))
+        mock_action.assert_called_once_with(
+            self.ctx,
+            '12345678ABC', 'CLUSTER_CREATE',
+            name='cluster_create_12345678',
+            cause=consts.CAUSE_RPC,
+            status=am.Action.READY,
+        )
+        notify.assert_called_once_with()
+
+    @mock.patch.object(service.EngineService, 'check_cluster_quota')
+    @mock.patch.object(su, 'check_size_params')
+    @mock.patch.object(am.Action, 'create')
+    @mock.patch.object(co.Cluster, "create")
+    @mock.patch.object(po.Profile, 'find')
+    @mock.patch.object(dispatcher, 'start_action')
+    def test_cluster_create_desired_null(self, notify, mock_profile,
+                                         mock_cluster, mock_action,
+                                         mock_check, mock_quota):
+        x_profile = mock.Mock(id='PROFILE_ID')
+        mock_profile.return_value = x_profile
+        x_cluster = mock.Mock(id='12345678ABC')
+        x_cluster.to_dict.return_value = {'foo': 'bar'}
+        mock_cluster.return_value = x_cluster
+        mock_action.return_value = 'ACTION_ID'
+        mock_check.return_value = None
+        mock_quota.return_value = None
+        req = orco.ClusterCreateRequestBody(name='C1', profile_id='PROFILE',
+                                            min_size=1, max_size=5,
+                                            config={'k1': 'v1'})
+
+        # do it
+        result = self.eng.cluster_create(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'action': 'ACTION_ID', 'foo': 'bar'}, result)
+        mock_profile.assert_called_once_with(self.ctx, 'PROFILE')
+        mock_check.assert_called_once_with(None, 1, 1, 5, True)
+        mock_cluster.assert_called_once_with(
+            self.ctx,
+            dict(name='C1', desired_capacity=1, profile_id='PROFILE_ID',
+                 min_size=1, max_size=5, timeout=3600, metadata={},
+                 dependents={}, data={}, next_index=1, status='INIT',
+                 config={'k1': 'v1'},
                  status_reason='Initializing', user=self.ctx.user,
                  project=self.ctx.project, domain=self.ctx.domain))
         mock_action.assert_called_once_with(
@@ -175,7 +218,7 @@ class ClusterTest(base.SenlinTestCase):
     @mock.patch.object(service.EngineService, 'check_cluster_quota')
     @mock.patch.object(co.Cluster, 'get_by_name')
     def test_cluster_create_duplicate_name(self, mock_get, mock_quota):
-        cfg.CONF.set_override('name_unique', True, enforce_type=True)
+        cfg.CONF.set_override('name_unique', True)
         mock_quota.return_value = None
         mock_get.return_value = mock.Mock()
         req = {'profile_id': 'PROFILE', 'name': 'CLUSTER'}
@@ -267,7 +310,8 @@ class ClusterTest(base.SenlinTestCase):
         mock_action.return_value = 'ACTION_ID'
         req = orco.ClusterUpdateRequest(identity='FAKE_ID', name='new_name',
                                         profile_id='NEW_PROFILE',
-                                        metadata={'B': 'A'}, timeout=120)
+                                        metadata={'B': 'A'}, timeout=120,
+                                        config={'k1': 'v1'})
 
         # do it
         result = self.eng.cluster_update(self.ctx, req.obj_to_primitive())
@@ -290,6 +334,9 @@ class ClusterTest(base.SenlinTestCase):
                 },
                 'timeout': 120,
                 'name': 'new_name',
+                'config': {
+                    'k1': 'v1',
+                },
             }
         )
 
@@ -1394,7 +1441,8 @@ class ClusterTest(base.SenlinTestCase):
         x_cluster = mock.Mock(id='CID')
         mock_find.return_value = x_cluster
         mock_action.return_value = 'ACTION_ID'
-        req = orco.ClusterRecoverRequest(identity='C1', params={'foo': 'bar'})
+        req = orco.ClusterRecoverRequest(identity='C1',
+                                         params={'operation': 'RECREATE'})
 
         result = self.eng.cluster_recover(self.ctx, req.obj_to_primitive())
 
@@ -1405,7 +1453,7 @@ class ClusterTest(base.SenlinTestCase):
             name='cluster_recover_CID',
             cause=consts.CAUSE_RPC,
             status=am.Action.READY,
-            inputs={'foo': 'bar'},
+            inputs={'operation': 'RECREATE'},
         )
         notify.assert_called_once_with()
 
@@ -1424,6 +1472,23 @@ class ClusterTest(base.SenlinTestCase):
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
+    @mock.patch.object(co.Cluster, 'find')
+    def test_cluster_recover_invalid(self, mock_find):
+        x_cluster = mock.Mock(id='CID')
+        mock_find.return_value = x_cluster
+
+        req = orco.ClusterRecoverRequest(identity='Bogus',
+                                         params={'bad': 'fake'})
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.cluster_recover,
+                               self.ctx, req.obj_to_primitive())
+
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual("Action parameter ['bad'] is not recognizable.",
+                         six.text_type(ex.exc_info[1]))
+        mock_find.assert_called_once_with(self.ctx, 'Bogus')
+
     @mock.patch.object(am.Action, 'create')
     @mock.patch.object(co.Cluster, 'find')
     @mock.patch.object(dispatcher, 'start_action')
@@ -1686,7 +1751,7 @@ class ClusterTest(base.SenlinTestCase):
         mock_chk.assert_called_once_with(self.ctx, cluster, nodes)
 
     @mock.patch.object(nm.Node, 'load')
-    @mock.patch.object(no.Node, 'get_all')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(co.Cluster, 'find')
     def test_cluster_collect(self, mock_find, mock_get, mock_load):
         x_cluster = mock.Mock(id='FAKE_CLUSTER')
@@ -1712,7 +1777,7 @@ class ClusterTest(base.SenlinTestCase):
         self.assertIn({'id': 'NODE2', 'value': '5.6.7.8'},
                       res['cluster_attributes'])
         mock_find.assert_called_once_with(self.ctx, 'CLUSTER_ID')
-        mock_get.assert_called_once_with(self.ctx, cluster_id='FAKE_CLUSTER')
+        mock_get.assert_called_once_with(self.ctx, 'FAKE_CLUSTER')
         mock_load.assert_has_calls([
             mock.call(self.ctx, db_node=x_obj_1),
             mock.call(self.ctx, db_node=x_obj_2)
@@ -1736,7 +1801,7 @@ class ClusterTest(base.SenlinTestCase):
         mock_parser.assert_called_once_with('foo.bar')
         self.assertEqual(0, mock_find.call_count)
 
-    @mock.patch.object(no.Node, 'get_all')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(co.Cluster, 'find')
     def test_cluster_collect_cluster_not_found(self, mock_find, mock_get):
         cid = 'FAKE_CLUSTER'
@@ -1751,7 +1816,7 @@ class ClusterTest(base.SenlinTestCase):
         mock_find.assert_called_once_with(self.ctx, cid)
         self.assertEqual(0, mock_get.call_count)
 
-    @mock.patch.object(no.Node, 'get_all')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(co.Cluster, 'find')
     def test_cluster_collect_no_nodes(self, mock_find, mock_get):
         x_cluster = mock.Mock(id='FAKE_CLUSTER')
@@ -1763,9 +1828,9 @@ class ClusterTest(base.SenlinTestCase):
 
         self.assertEqual({'cluster_attributes': []}, res)
         mock_find.assert_called_once_with(self.ctx, 'CLUSTER_ID')
-        mock_get.assert_called_once_with(self.ctx, cluster_id='FAKE_CLUSTER')
+        mock_get.assert_called_once_with(self.ctx, 'FAKE_CLUSTER')
 
-    @mock.patch.object(no.Node, 'get_all')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(co.Cluster, 'find')
     def test_cluster_collect_no_details(self, mock_find, mock_get):
         x_cluster = mock.Mock(id='FAKE_CLUSTER')
@@ -1785,13 +1850,13 @@ class ClusterTest(base.SenlinTestCase):
         self.assertIn({'id': 'NODE2', 'value': 'node2'},
                       res['cluster_attributes'])
         mock_find.assert_called_once_with(self.ctx, 'CLUSTER_ID')
-        mock_get.assert_called_once_with(self.ctx, cluster_id='FAKE_CLUSTER')
+        mock_get.assert_called_once_with(self.ctx, 'FAKE_CLUSTER')
         x_node_1.to_dict.assert_called_once_with()
         self.assertEqual(0, x_node_1.get_details.call_count)
         x_node_2.to_dict.assert_called_once_with()
         self.assertEqual(0, x_node_2.get_details.call_count)
 
-    @mock.patch.object(no.Node, 'get_all')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(co.Cluster, 'find')
     def test_cluster_collect_no_match(self, mock_find, mock_get):
         x_cluster = mock.Mock(id='FAKE_CLUSTER')
@@ -1807,7 +1872,7 @@ class ClusterTest(base.SenlinTestCase):
 
         self.assertEqual({'cluster_attributes': []}, res)
         mock_find.assert_called_once_with(self.ctx, 'CLUSTER_ID')
-        mock_get.assert_called_once_with(self.ctx, cluster_id='FAKE_CLUSTER')
+        mock_get.assert_called_once_with(self.ctx, 'FAKE_CLUSTER')
         x_node_1.to_dict.assert_called_once_with()
         self.assertEqual(0, x_node_1.get_details.call_count)
         x_node_2.to_dict.assert_called_once_with()
