@@ -20,12 +20,14 @@ from senlin.common import exception as exc
 from senlin.common.i18n import _
 from senlin.engine.actions import base as action_mod
 from senlin.engine import dispatcher
+from senlin.engine import environment
 from senlin.engine import node as node_mod
 from senlin.engine import service
 from senlin.objects import cluster as co
 from senlin.objects import node as no
 from senlin.objects import profile as po
 from senlin.objects.requests import nodes as orno
+from senlin.profiles import base as pb
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -296,7 +298,7 @@ class NodeTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'get_by_name')
     def test_node_create_name_conflict(self, mock_get):
-        cfg.CONF.set_override('name_unique', True, enforce_type=True)
+        cfg.CONF.set_override('name_unique', True)
         mock_get.return_value = mock.Mock()
         req = orno.NodeCreateRequestBody(name='NODE1',
                                          profile_id='PROFILE_NAME')
@@ -648,6 +650,190 @@ class NodeTest(base.SenlinTestCase):
                          "by other clusters and/or nodes.",
                          six.text_type(ex.exc_info[1]))
 
+    @mock.patch.object(environment.Environment, 'get_profile')
+    @mock.patch.object(pb.Profile, 'adopt_node')
+    def test__node_adopt_preview(self, mock_adopt, mock_profile):
+        class FakeProfile(object):
+            pass
+
+        req = mock.Mock(
+            identity="FAKE_NODE",
+            type="TestProfile-1.0",
+            overrides="foo",
+            snapshot=True
+        )
+        mock_adopt.return_value = {'prop': 'value'}
+        mock_profile.return_value = FakeProfile
+
+        c, s = self.eng._node_adopt_preview(self.ctx, req)
+
+        req.obj_set_defaults.assert_called_once_with()
+        mock_profile.assert_called_once_with("TestProfile-1.0")
+        self.assertEqual(FakeProfile, c)
+        mock_adopt.assert_called_once_with(
+            self.ctx, mock.ANY, 'TestProfile-1.0',
+            overrides="foo", snapshot=True)
+        fake_node = mock_adopt.call_args[0][1]
+        self.assertIsInstance(fake_node, node_mod.Node)
+        self.assertEqual('adopt', fake_node.name)
+        self.assertEqual('TBD', fake_node.profile_id)
+        self.assertEqual('FAKE_NODE', fake_node.physical_id)
+        expected = {
+            'type': 'TestProfile',
+            'version': '1.0',
+            'properties': {'prop': 'value'}
+        }
+        self.assertEqual(expected, s)
+
+    @mock.patch.object(pb.Profile, 'adopt_node')
+    def test__node_adopt_preview_bad_type(self, mock_adopt):
+        req = mock.Mock(
+            identity="FAKE_NODE",
+            type="TestProfile-1.0",
+            overrides="foo",
+            snapshot=True
+        )
+
+        ex = self.assertRaises(exc.BadRequest,
+                               self.eng._node_adopt_preview,
+                               self.ctx, req)
+
+        req.obj_set_defaults.assert_called_once_with()
+        self.assertEqual("The profile_type 'TestProfile-1.0' could not be "
+                         "found.", six.text_type(ex))
+
+    @mock.patch.object(environment.Environment, 'get_profile')
+    @mock.patch.object(pb.Profile, 'adopt_node')
+    def test__node_adopt_preview_failed_adopt(self, mock_adopt, mock_profile):
+        class FakeProfile(object):
+            pass
+
+        req = mock.Mock(
+            identity="FAKE_NODE",
+            type="TestProfile-1.0",
+            overrides="foo",
+            snapshot=True
+        )
+        mock_profile.return_value = FakeProfile
+        mock_adopt.return_value = {
+            'Error': {'code': 502, 'message': 'something is bad'}
+        }
+
+        ex = self.assertRaises(exc.ProfileOperationFailed,
+                               self.eng._node_adopt_preview,
+                               self.ctx, req)
+
+        req.obj_set_defaults.assert_called_once_with()
+        mock_profile.assert_called_once_with("TestProfile-1.0")
+        mock_adopt.assert_called_once_with(
+            self.ctx, mock.ANY, 'TestProfile-1.0',
+            overrides="foo", snapshot=True)
+
+        self.assertEqual('502: something is bad', six.text_type(ex))
+
+    @mock.patch.object(service.EngineService, '_node_adopt_preview')
+    def test_node_adopt_preview(self, mock_preview):
+        spec = {'foo': 'bar'}
+        mock_preview.return_value = mock.Mock(), spec
+        req = orno.NodeAdoptPreviewRequest(identity='FAKE_ID',
+                                           type='FAKE_TYPE')
+
+        res = self.eng.node_adopt_preview(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'node_preview': {'foo': 'bar'}}, res)
+        mock_preview.assert_called_once_with(self.ctx, mock.ANY)
+        self.assertIsInstance(mock_preview.call_args[0][1],
+                              orno.NodeAdoptPreviewRequest)
+
+    @mock.patch.object(service.EngineService, '_node_adopt_preview')
+    def test_node_adopt_preview_with_exception(self, mock_preview):
+        mock_preview.side_effect = exc.BadRequest(msg="boom")
+        req = orno.NodeAdoptPreviewRequest(identity='FAKE_ID',
+                                           type='FAKE_TYPE')
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.node_adopt_preview,
+                               self.ctx, req.obj_to_primitive())
+
+        mock_preview.assert_called_once_with(self.ctx, mock.ANY)
+        self.assertIsInstance(mock_preview.call_args[0][1],
+                              orno.NodeAdoptPreviewRequest)
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual('boom.', six.text_type(ex.exc_info[1]))
+
+    @mock.patch.object(no.Node, 'create')
+    @mock.patch.object(service.EngineService, '_node_adopt_preview')
+    def test_node_adopt(self, mock_preview, mock_create):
+        class FakeProfile(object):
+            @classmethod
+            def create(cls, ctx, name, spec):
+                obj = mock.Mock(spec=spec, id='PROFILE_ID')
+                obj.name = name
+                return obj
+
+        req = orno.NodeAdoptRequest(identity='FAKE_ID', type='FAKE_TYPE')
+        mock_preview.return_value = FakeProfile, {'foo': 'bar'}
+        fake_node = mock.Mock()
+        fake_node.to_dict = mock.Mock(return_value={'attr': 'value'})
+        mock_create.return_value = fake_node
+
+        res = self.eng.node_adopt(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'attr': 'value'}, res)
+        mock_preview.assert_called_once_with(self.ctx, mock.ANY)
+        self.assertIsInstance(mock_preview.call_args[0][1],
+                              orno.NodeAdoptRequest)
+        attrs = {
+            'name': mock.ANY,
+            'profile_id': 'PROFILE_ID',
+            'physical_id': 'FAKE_ID',
+            'cluster_id': '',
+            'index': -1,
+            'role': '',
+            'metadata': {},
+            'status': consts.NS_ACTIVE,
+            'status_reason': 'Node adopted successfully',
+            'init_at': mock.ANY,
+            'created_at': mock.ANY,
+            'user': self.ctx.user,
+            'project': self.ctx.project,
+            'domain': self.ctx.domain
+        }
+        mock_create.assert_called_once_with(self.ctx, attrs)
+
+    @mock.patch.object(no.Node, 'get_by_name')
+    def test_node_adopt_name_not_unique(self, mock_get):
+        cfg.CONF.set_override('name_unique', True)
+        req = orno.NodeAdoptRequest(
+            name='FAKE_NAME', preview=False,
+            identity='FAKE_ID', type='FAKE_TYPE')
+        mock_get.return_value = mock.Mock()
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.node_adopt,
+                               self.ctx, req.obj_to_primitive())
+
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual("The node named (FAKE_NAME) already exists.",
+                         six.text_type(ex.exc_info[1]))
+
+    @mock.patch.object(no.Node, 'create')
+    @mock.patch.object(service.EngineService, '_node_adopt_preview')
+    def test_node_adopt_failed_preview(self, mock_preview, mock_create):
+        req = orno.NodeAdoptRequest(identity='FAKE_ID', type='FAKE_TYPE')
+        mock_preview.side_effect = exc.BadRequest(msg='boom')
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.node_adopt,
+                               self.ctx, req.obj_to_primitive())
+
+        mock_preview.assert_called_once_with(self.ctx, mock.ANY)
+        self.assertIsInstance(mock_preview.call_args[0][1],
+                              orno.NodeAdoptRequest)
+
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual("boom.", six.text_type(ex.exc_info[1]))
+
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch.object(no.Node, 'find')
@@ -690,7 +876,7 @@ class NodeTest(base.SenlinTestCase):
         mock_find.return_value = mock.Mock(id='12345678AB')
         mock_action.return_value = 'ACTION_ID'
 
-        params = {'k1': 'v1'}
+        params = {'operation': 'some_action'}
         req = orno.NodeRecoverRequest(identity='FAKE_NODE', params=params)
         result = self.eng.node_recover(self.ctx, req.obj_to_primitive())
 
@@ -701,7 +887,28 @@ class NodeTest(base.SenlinTestCase):
             name='node_recover_12345678',
             cause=consts.CAUSE_RPC,
             status=action_mod.Action.READY,
-            inputs={'k1': 'v1'})
+            inputs={'operation': [{'name': 'some_action'}]})
+        mock_start.assert_called_once_with()
+
+    @mock.patch.object(dispatcher, 'start_action')
+    @mock.patch.object(action_mod.Action, 'create')
+    @mock.patch.object(no.Node, 'find')
+    def test_node_recover_with_check(self, mock_find, mock_action, mock_start):
+        mock_find.return_value = mock.Mock(id='12345678AB')
+        mock_action.return_value = 'ACTION_ID'
+
+        params = {'check': True, 'operation': 'some_action'}
+        req = orno.NodeRecoverRequest(identity='FAKE_NODE', params=params)
+        result = self.eng.node_recover(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'action': 'ACTION_ID'}, result)
+        mock_find.assert_called_once_with(self.ctx, 'FAKE_NODE')
+        mock_action.assert_called_once_with(
+            self.ctx, '12345678AB', consts.NODE_RECOVER,
+            name='node_recover_12345678',
+            cause=consts.CAUSE_RPC,
+            status=action_mod.Action.READY,
+            inputs={'check': True, 'operation': [{'name': 'some_action'}]})
         mock_start.assert_called_once_with()
 
     @mock.patch.object(no.Node, 'find')
@@ -718,6 +925,24 @@ class NodeTest(base.SenlinTestCase):
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
+    @mock.patch.object(action_mod.Action, 'create')
+    @mock.patch.object(no.Node, 'find')
+    def test_node_recover_invalid_operation(self, mock_find, mock_action):
+        mock_find.return_value = mock.Mock(id='12345678AB')
+        mock_action.return_value = 'ACTION_ID'
+        params = {'bogus': 'illegal'}
+        req = orno.NodeRecoverRequest(identity='FAKE_NODE', params=params)
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.node_recover,
+                               self.ctx, req.obj_to_primitive())
+
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual("Action parameter ['bogus'] is not recognizable.",
+                         six.text_type(ex.exc_info[1]))
+        mock_find.assert_called_once_with(self.ctx, 'FAKE_NODE')
+        self.assertEqual(0, mock_action.call_count)
+
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch.object(node_mod.Node, 'load')
