@@ -22,6 +22,7 @@ from senlin.engine import health_manager
 from senlin.engine import node as node_mod
 from senlin.objects import cluster as co
 from senlin.objects import cluster_policy as cpo
+from senlin.objects import node as no
 from senlin.policies import base as pcb
 from senlin.profiles import base as pfb
 from senlin.tests.unit.common import base
@@ -61,6 +62,8 @@ class TestCluster(base.SenlinTestCase):
         self.assertEqual('Initializing', cluster.status_reason)
         self.assertEqual({}, cluster.data)
         self.assertEqual({}, cluster.metadata)
+        self.assertEqual({}, cluster.dependents)
+        self.assertEqual({}, cluster.config)
         self.assertEqual({'profile': None, 'nodes': [], 'policies': []},
                          cluster.rt)
 
@@ -83,7 +86,7 @@ class TestCluster(base.SenlinTestCase):
     @mock.patch.object(cpo.ClusterPolicy, 'get_all')
     @mock.patch.object(pcb.Policy, 'load')
     @mock.patch.object(pfb.Profile, 'load')
-    @mock.patch.object(node_mod.Node, 'load_all')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     def test__load_runtime_data(self, mock_nodes, mock_profile, mock_policy,
                                 mock_pb):
         x_binding = mock.Mock()
@@ -106,7 +109,7 @@ class TestCluster(base.SenlinTestCase):
         self.assertEqual(x_profile, rt['profile'])
         self.assertEqual([x_node_1, x_node_2], rt['nodes'])
         self.assertEqual(2, len(rt['nodes']))
-        self.assertTrue(isinstance(rt['nodes'], list))
+        self.assertIsInstance(rt['nodes'], list)
         self.assertEqual([x_policy], rt['policies'])
 
         mock_pb.assert_called_once_with(self.context, CLUSTER_ID)
@@ -116,8 +119,7 @@ class TestCluster(base.SenlinTestCase):
         mock_profile.assert_called_once_with(self.context,
                                              profile_id=PROFILE_ID,
                                              project_safe=False)
-        mock_nodes.assert_called_once_with(self.context,
-                                           cluster_id=CLUSTER_ID)
+        mock_nodes.assert_called_once_with(self.context, CLUSTER_ID)
 
     def test__load_runtime_data_id_is_none(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
@@ -128,7 +130,7 @@ class TestCluster(base.SenlinTestCase):
         self.assertIsNone(rt['profile'])
         self.assertEqual([], rt['nodes'])
         self.assertEqual(0, len(rt['nodes']))
-        self.assertTrue(isinstance(rt['nodes'], list))
+        self.assertIsInstance(rt['nodes'], list)
         self.assertEqual([], rt['policies'])
 
     def test_store_for_create(self):
@@ -186,6 +188,7 @@ class TestCluster(base.SenlinTestCase):
         cluster.timeout = 120
         cluster.data = {'FOO': 'BAR'}
         cluster.metadata = {'KEY': 'VALUE'}
+        cluster.config = {'KEY': 'VALUE'}
 
         new_id = cluster.store(self.context)
         self.assertEqual(cluster_id, new_id)
@@ -204,6 +207,7 @@ class TestCluster(base.SenlinTestCase):
         self.assertEqual(120, result.timeout)
         self.assertEqual({'FOO': 'BAR'}, result.data)
         self.assertEqual({'KEY': 'VALUE'}, result.metadata)
+        self.assertEqual({'KEY': 'VALUE'}, result.config)
 
     @mock.patch.object(cm.Cluster, '_from_object')
     def test_load_via_db_object(self, mock_init):
@@ -489,6 +493,24 @@ class TestCluster(base.SenlinTestCase):
         self.assertIsNone(res)
         self.assertEqual([], cluster.nodes)
 
+    def test_update_node(self):
+        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
+        self.assertEqual([], cluster.nodes)
+
+        node1 = mock.Mock(id='fake', status='ACTIVE')
+        # add one
+        cluster.add_node(node1)
+
+        node1.status = 'ERROR'
+        cluster.update_node([node1])
+        self.assertEqual([node1], cluster.nodes)
+
+        # update new ones
+        node2 = mock.Mock(id='fake1', status='ACTIVE')
+        node3 = mock.Mock(id='fake2', status='ERROR')
+        cluster.update_node([node2, node3])
+        self.assertEqual([node2, node3], cluster.nodes)
+
     @mock.patch.object(pcb.Policy, 'load')
     @mock.patch.object(cpm, 'ClusterPolicy')
     def test_attach_policy(self, mock_cp, mock_load):
@@ -668,7 +690,7 @@ class TestCluster(base.SenlinTestCase):
 
         existing = mock.Mock()
         existing.id = POLICY_ID
-        existing.type = "senlin.poicy.foo"
+        existing.type = "senlin.policy.foo"
         cluster.rt['policies'] = [existing]
         values = {
             'enabled': False
@@ -837,6 +859,33 @@ class TestCluster(base.SenlinTestCase):
         result = cluster.nodes_by_region('AZ3')
         self.assertEqual(0, len(result))
 
+    @mock.patch.object(node_mod.Node, 'load_all')
+    @mock.patch.object(node_mod.Node, 'do_check')
+    @mock.patch.object(cm.Cluster, 'update_node')
+    def test_health_check(self, mock_update, mock_check, mock_load):
+        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID,
+                             min_size=2, id=CLUSTER_ID)
+        node1 = node_mod.Node('fake1', PROFILE_ID, status='ACTIVE')
+        node2 = node_mod.Node('fake2', PROFILE_ID, status='ACTIVE')
+        nodes = [node1, node2]
+        for node in nodes:
+            cluster.add_node(node)
+
+        node1.status = 'ERROR'
+        mock_load.return_value = [node1, node2]
+
+        cluster.health_check(self.context)
+
+        self.assertEqual(2, len(cluster.nodes))
+        self.assertEqual([node1, node2], cluster.nodes)
+
+        mock_update.assert_called_once_with([node1, node2])
+        mock_check.assert_has_calls([
+            mock.call(self.context),
+            mock.call(self.context)
+        ])
+        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
+
     @mock.patch.object(co.Cluster, 'update')
     @mock.patch.object(node_mod.Node, 'load_all')
     def test_eval_status_below_min_size(self, mock_load, mock_update):
@@ -850,7 +899,7 @@ class TestCluster(base.SenlinTestCase):
         cluster.eval_status(self.context, 'TEST')
         rt = cluster.rt
         self.assertEqual(3, len(rt['nodes']))
-        self.assertTrue(isinstance(rt['nodes'], list))
+        self.assertIsInstance(rt['nodes'], list)
         mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
