@@ -583,9 +583,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc.stack_check = mock.Mock(
             side_effect=exc.InternalError(code=400, message='BOOM'))
 
-        res = profile.do_check(node_obj)
+        self.assertRaises(exc.EResourceOperation, profile.do_check, node_obj)
 
-        self.assertFalse(res)
         oc.stack_check.assert_called_once_with('FAKE_ID')
         self.assertEqual(0, oc.wait_for_stack.call_count)
 
@@ -597,9 +596,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc.wait_for_stack = mock.Mock(
             side_effect=exc.InternalError(code=400, message='BOOM'))
 
-        res = profile.do_check(node_obj)
+        self.assertRaises(exc.EResourceOperation, profile.do_check, node_obj)
 
-        self.assertFalse(res)
         oc.stack_check.assert_called_once_with('FAKE_ID')
         oc.wait_for_stack.assert_called_once_with(
             'FAKE_ID', 'CHECK_COMPLETE', timeout=3600)
@@ -638,11 +636,144 @@ class TestHeatStackProfile(base.SenlinTestCase):
         self.assertEqual({'Error': {'code': 500, 'message': 'BOOM'}}, res)
         oc.stack_get.assert_called_once_with('STACK_ID')
 
+    def test_do_adopt(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock(
+            parameters={'p1': 'v1', 'OS::stack_id': 'FAKE_ID'},
+            timeout_mins=123,
+            disable_rollback=False
+        )
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template = mock.Mock(return_value={'foo': 'bar'})
+        oc.stack_get_environment = mock.Mock(return_value={'ke': 've'})
+        oc.stack_get_files = mock.Mock(return_value={'fn': 'content'})
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {
+            'environment': {'ke': 've'},
+            'files': {'fn': 'content'},
+            'template': {'foo': 'bar'},
+            'parameters': {'p1': 'v1'},
+            'timeout': 123,
+            'disable_rollback': False
+        }
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+        oc.stack_get_environment.assert_called_once_with('FAKE_ID')
+        oc.stack_get_files.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        oc.stack_get.side_effect = exc.InternalError(message='BOOM')
+        profile._orchestrationclient = oc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get_template(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock()
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template.side_effect = exc.InternalError(message='BOOM')
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get_environment(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock()
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template = mock.Mock(return_value={'foo': 'bar'})
+        err = exc.InternalError(message='BOOM')
+        oc.stack_get_environment.side_effect = err
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+        oc.stack_get_environment.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get_files(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock()
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template = mock.Mock(return_value={'foo': 'bar'})
+        oc.stack_get_environment = mock.Mock(return_value={'ke': 've'})
+        oc.stack_get_files.side_effect = exc.InternalError(message='BOOM')
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+        oc.stack_get_environment.assert_called_once_with('FAKE_ID')
+        oc.stack_get_files.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_with_overrides(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock(
+            parameters={'p1': 'v1', 'OS::stack_id': 'FAKE_ID'},
+            timeout_mins=123,
+            disable_rollback=False
+        )
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template = mock.Mock(return_value={'foo': 'bar'})
+        oc.stack_get_environment = mock.Mock(return_value={'ke': 've'})
+        oc.stack_get_files = mock.Mock(return_value={'fn': 'content'})
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+        overrides = {'environment': {'ENV': 'SETTING'}}
+        res = profile.do_adopt(node_obj, overrides=overrides)
+
+        expected = {
+            'environment': {'ENV': 'SETTING'},
+            'files': {'fn': 'content'},
+            'template': {'foo': 'bar'},
+            'parameters': {'p1': 'v1'},
+            'timeout': 123,
+            'disable_rollback': False
+        }
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+
     def test__refresh_tags_empty_no_add(self):
         profile = stack.StackProfile('t', self.spec)
         node = mock.Mock()
 
-        res = profile._refresh_tags("", node, False)
+        res = profile._refresh_tags([], node, False)
 
         self.assertEqual(("", False), res)
 
@@ -650,7 +781,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         profile = stack.StackProfile('t', self.spec)
         node = mock.Mock()
 
-        res = profile._refresh_tags('foo', node, False)
+        res = profile._refresh_tags(['foo'], node, False)
 
         self.assertEqual(('foo', False), res)
 
@@ -658,7 +789,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         profile = stack.StackProfile('t', self.spec)
         node = mock.Mock()
 
-        res = profile._refresh_tags('cluster_id=FOO,bar', node, False)
+        res = profile._refresh_tags(['cluster_id=FOO', 'bar'], node, False)
 
         self.assertEqual(('bar', True), res)
 
@@ -666,7 +797,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         profile = stack.StackProfile('t', self.spec)
         node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
 
-        res = profile._refresh_tags("", node, True)
+        res = profile._refresh_tags([], node, True)
 
         expected = ",".join(['cluster_id=CLUSTER_ID',
                              'cluster_node_id=NODE_ID',
@@ -677,7 +808,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         profile = stack.StackProfile('t', self.spec)
         node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
 
-        res = profile._refresh_tags('foo', node, True)
+        res = profile._refresh_tags(['foo'], node, True)
 
         expected = ",".join(['foo',
                              'cluster_id=CLUSTER_ID',
@@ -689,7 +820,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         profile = stack.StackProfile('t', self.spec)
         node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
 
-        res = profile._refresh_tags('cluster_id=FOO,bar', node, True)
+        res = profile._refresh_tags(['cluster_id=FOO', 'bar'], node, True)
 
         expected = ",".join(['bar',
                              'cluster_id=CLUSTER_ID',
@@ -712,7 +843,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         self.assertTrue(res)
         oc.stack_get.assert_called_once_with('STACK_ID')
         mock_tags.assert_called_once_with('foo', node, True)
-        oc.stack_update.assert_called_once_with('STACK_ID', {'tags': 'bar'})
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
 
     def test_do_join_no_physical_id(self):
         profile = stack.StackProfile('t', self.spec)
@@ -769,7 +900,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         self.assertFalse(res)
         oc.stack_get.assert_called_once_with('STACK_ID')
         mock_tags.assert_called_once_with('foo', node, True)
-        oc.stack_update.assert_called_once_with('STACK_ID', {'tags': 'bar'})
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
 
     def test_do_leave(self):
         profile = stack.StackProfile('t', self.spec)
@@ -786,7 +917,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         self.assertTrue(res)
         oc.stack_get.assert_called_once_with('STACK_ID')
         mock_tags.assert_called_once_with('foo', node, False)
-        oc.stack_update.assert_called_once_with('STACK_ID', {'tags': 'bar'})
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
 
     def test_do_leave_no_physical_id(self):
         profile = stack.StackProfile('t', self.spec)
@@ -843,4 +974,4 @@ class TestHeatStackProfile(base.SenlinTestCase):
         self.assertFalse(res)
         oc.stack_get.assert_called_once_with('STACK_ID')
         mock_tags.assert_called_once_with('foo', node, False)
-        oc.stack_update.assert_called_once_with('STACK_ID', {'tags': 'bar'})
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
