@@ -17,6 +17,7 @@ from oslo_utils import encodeutils
 import six
 
 from senlin.common import exception as exc
+from senlin.objects import node as node_ob
 from senlin.profiles import base as profiles_base
 from senlin.profiles.os.nova import server
 from senlin.tests.unit.common import base
@@ -44,9 +45,9 @@ class TestNovaServerBasic(base.SenlinTestCase):
                 "metadata": {"meta var": "meta val"},
                 'name': 'FAKE_SERVER_NAME',
                 'networks': [{
-                    'port': 'FAKE_PORT',
                     'fixed_ip': 'FAKE_IP',
                     'network': 'FAKE_NET',
+                    'floating_network': 'FAKE_PUBLIC_NET',
                 }],
                 'personality': [{
                     'path': '/etc/motd',
@@ -116,9 +117,15 @@ class TestNovaServerBasic(base.SenlinTestCase):
                 'fixed_ip': 'FAKE_IP',
                 'port': 'FAKE_PORT',
                 'uuid': 'FAKE_NETWORK_ID',
+                'floating_network': 'FAKE_PUBLIC_NET_ID',
             }
             self.patchobject(profile, '_validate_network',
                              return_value=fake_net)
+            fake_ports = [{
+                'id': 'FAKE_PORT'
+            }]
+            self.patchobject(profile, '_create_ports_from_properties',
+                             return_value=fake_ports)
 
     def test_do_create(self):
         cc = mock.Mock()
@@ -140,6 +147,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         node_obj.name = 'TEST_SERVER'
         fake_server = mock.Mock(id='FAKE_ID')
         cc.server_create.return_value = fake_server
+        cc.server_get.return_value = fake_server
 
         # do it
         server_id = profile.do_create(node_obj)
@@ -160,9 +168,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
             },
             name='FAKE_SERVER_NAME',
             networks=[{
-                'fixed_ip': 'FAKE_IP',
                 'port': 'FAKE_PORT',
-                'uuid': 'FAKE_NETWORK_ID',
             }],
             personality=[{
                 'path': '/etc/motd',
@@ -181,6 +187,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         attrs['OS-DCF:diskConfig'] = 'AUTO'
 
         cc.server_create.assert_called_once_with(**attrs)
+        cc.server_get.assert_called_once_with('FAKE_ID')
         mock_zone_info.assert_called_once_with(node_obj, fake_server)
         self.assertEqual('FAKE_ID', server_id)
 
@@ -250,9 +257,16 @@ class TestNovaServerBasic(base.SenlinTestCase):
         self.assertRaises(exc.EResourceCreation,
                           profile.do_create,
                           node_obj)
+        expect_params = {
+            'floating_network': None,
+            'network': 'FAKE_NET',
+            'fixed_ip': None,
+            'floating_ip': None,
+            'port': None,
+            'security_groups': None
+        }
         mock_net.assert_called_once_with(
-            node_obj, {'network': 'FAKE_NET', 'port': None, 'fixed_ip': None},
-            'create')
+            node_obj, expect_params, 'create')
 
     def test_do_create_server_attrs_not_defined(self):
         cc = mock.Mock()
@@ -278,6 +292,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         mock_zone_info = self.patchobject(profile, '_update_zone_info')
         fake_server = mock.Mock(id='FAKE_ID')
         cc.server_create.return_value = fake_server
+        cc.server_get.return_value = fake_server
 
         # do it
         server_id = profile.do_create(node_obj)
@@ -296,6 +311,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         }
 
         cc.server_create.assert_called_once_with(**attrs)
+        cc.server_get.assert_called_once_with('FAKE_ID')
         mock_zone_info.assert_called_once_with(node_obj, fake_server)
         self.assertEqual('FAKE_ID', server_id)
 
@@ -322,6 +338,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         node_obj.name = None
         fake_server = mock.Mock(id='FAKE_ID')
         cc.server_create.return_value = fake_server
+        cc.server_get.return_value = fake_server
 
         server_id = profile.do_create(node_obj)
 
@@ -334,6 +351,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         }
 
         cc.server_create.assert_called_once_with(**attrs)
+        cc.server_get.assert_called_once_with('FAKE_ID')
         mock_zone_info.assert_called_once_with(node_obj, fake_server)
         self.assertEqual('FAKE_ID', server_id)
 
@@ -359,6 +377,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         node_obj.name = 'TEST-SERVER'
         fake_server = mock.Mock(id='FAKE_ID')
         cc.server_create.return_value = fake_server
+        cc.server_get.return_value = fake_server
 
         # do it
         server_id = profile.do_create(node_obj)
@@ -373,6 +392,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         }
 
         cc.server_create.assert_called_once_with(**attrs)
+        cc.server_get.assert_called_once_with('FAKE_ID')
         mock_zone_info.assert_called_once_with(node_obj, fake_server)
         self.assertEqual('FAKE_ID', server_id)
 
@@ -413,6 +433,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         node_obj.name = None
         fake_server = mock.Mock(id='FAKE_ID')
         cc.server_create.return_value = fake_server
+        cc.server_get.return_value = fake_server
 
         # do it
         server_id = profile.do_create(node_obj)
@@ -441,13 +462,16 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'block_device_mapping_v2': bdm_v2
         }
         cc.server_create.assert_called_once_with(**attrs)
+        cc.server_get.assert_called_once_with('FAKE_ID')
         mock_zone_info.assert_called_once_with(node_obj, fake_server)
         self.assertEqual('FAKE_ID', server_id)
 
     def test_do_create_wait_server_timeout(self):
         cc = mock.Mock()
+        nc = mock.Mock()
         profile = server.ServerProfile('t', self.spec)
         profile._computeclient = cc
+        profile._networkclient = nc
         self._stubout_profile(profile, mock_image=True, mock_flavor=True,
                               mock_keypair=True, mock_net=True)
 
@@ -509,6 +533,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         profile._computeclient = cc
 
         test_server = mock.Mock(physical_id='FAKE_ID')
+        test_server.data = {}
 
         res = profile.do_delete(test_server)
 
@@ -516,6 +541,39 @@ class TestNovaServerBasic(base.SenlinTestCase):
         cc.server_delete.assert_called_once_with('FAKE_ID', True)
         cc.wait_for_server_delete.assert_called_once_with('FAKE_ID')
 
+    @mock.patch.object(node_ob.Node, 'update')
+    def test_do_delete_ports_ok(self, mock_node_obj):
+        profile = server.ServerProfile('t', self.spec)
+
+        cc = mock.Mock()
+        cc.server_delete.return_value = None
+        nc = mock.Mock()
+        nc.port_delete.return_value = None
+        nc.floatingip_delete.return_value = None
+        profile._computeclient = cc
+        profile._networkclient = nc
+
+        test_server = mock.Mock(physical_id='FAKE_ID')
+        test_server.Node = mock.Mock()
+        test_server.data = {'internal_ports': [{
+            'floating': {
+                'remove': True,
+                'id': 'FAKE_FLOATING_ID',
+            },
+            'id': 'FAKE_PORT_ID',
+            'remove': True
+        }]}
+
+        res = profile.do_delete(test_server)
+
+        self.assertTrue(res)
+        mock_node_obj.assert_called_once_with(
+            mock.ANY, test_server.id, {'data': {'internal_ports': []}})
+        nc.floatingip_delete.assert_called_once_with('FAKE_FLOATING_ID')
+        nc.port_delete.assert_called_once_with('FAKE_PORT_ID')
+        cc.server_delete.assert_called_once_with('FAKE_ID', True)
+        cc.wait_for_server_delete.assert_called_once_with('FAKE_ID')
+
     def test_do_delete_ignore_missing_force(self):
         profile = server.ServerProfile('t', self.spec)
 
@@ -523,6 +581,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         profile._computeclient = cc
 
         test_server = mock.Mock(physical_id='FAKE_ID')
+        test_server.data = {}
 
         res = profile.do_delete(test_server, ignore_missing=False, force=True)
 
@@ -849,28 +908,126 @@ class TestNovaServerBasic(base.SenlinTestCase):
         self.assertEqual(expected, res)
         cc.server_get.assert_called_once_with('FAKE_ID')
 
-    def test_do_join_successful(self):
+    def test_do_adopt(self):
         profile = server.ServerProfile('t', self.spec)
-
-        cluster_id = "FAKE_CLUSTER_ID"
+        x_server = mock.Mock(
+            disk_config="",
+            availability_zone="AZ01",
+            block_device_mapping={"foo": "bar"},
+            has_config_drive=False,
+            flavor={"id": "FLAVOR_ID"},
+            image={"id": "IMAGE_ID"},
+            key_name="FAKE_KEY",
+            metadata={
+                "mkey": "mvalue",
+                "cluster_id": "CLUSTER_ID",
+                "cluster_node_id": "NODE_ID",
+                "cluster_node_index": 123
+            },
+            addresses={
+                "NET1": [{
+                    "OS-EXT-IPS:type": "fixed",
+                    "addr": "ADDR1"
+                }],
+                "NET2": [{
+                    "OS-EXT-IPS:type": "fixed",
+                    "addr": "ADDR2"
+                }],
+            },
+            security_groups=[{'name': 'GROUP1'}, {'name': 'GROUP2'}]
+        )
+        x_server.name = "FAKE_NAME"
         cc = mock.Mock()
-        cc.server_metadata_get.return_value = {'FOO': 'BAR'}
-        cc.server_metadata_update.return_value = {'cluster_id': cluster_id}
+        cc.server_get.return_value = x_server
         profile._computeclient = cc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
 
-        node_obj = mock.Mock(physical_id='FAKE_ID', index=567)
+        res = profile.do_adopt(node_obj)
 
-        res = profile.do_join(node_obj, cluster_id)
+        self.assertEqual(False, res['auto_disk_config'])
+        self.assertEqual('AZ01', res['availability_zone'])
+        self.assertEqual({'foo': 'bar'}, res['block_device_mapping_v2'])
+        self.assertFalse(res['config_drive'])
+        self.assertEqual('FLAVOR_ID', res['flavor'])
+        self.assertEqual('IMAGE_ID', res['image'])
+        self.assertEqual('FAKE_KEY', res['key_name'])
+        self.assertEqual({'mkey': 'mvalue'}, res['metadata'])
+        self.assertIn({'network': 'NET1'}, res['networks'])
+        self.assertIn({'network': 'NET2'}, res['networks'])
+        self.assertIn('GROUP1', res['security_groups'])
+        self.assertIn('GROUP2', res['security_groups'])
+        cc.server_get.assert_called_once_with('FAKE_ID')
 
-        self.assertTrue(res)
-        cc.server_metadata_get.assert_called_once_with('FAKE_ID')
-        expected_metadata = {
-            'cluster_id': 'FAKE_CLUSTER_ID',
-            'cluster_node_index': '567',
-            'FOO': 'BAR'
+    def test_do_adopt_failed_get(self):
+        profile = server.ServerProfile('t', self.spec)
+        cc = mock.Mock()
+        err = exc.InternalError(code=404, message='No Server found for ID')
+        cc.server_get.side_effect = err
+        profile._computeclient = cc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {
+            'Error': {
+                'code': 404,
+                'message': 'No Server found for ID',
+            }
         }
-        cc.server_metadata_update.assert_called_once_with(
-            'FAKE_ID', expected_metadata)
+        self.assertEqual(expected, res)
+        cc.server_get.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_with_overrides(self):
+        profile = server.ServerProfile('t', self.spec)
+        x_server = mock.Mock(
+            disk_config="",
+            availability_zone="AZ01",
+            block_device_mapping={"foo": "bar"},
+            has_config_drive=False,
+            flavor={"id": "FLAVOR_ID"},
+            image={"id": "IMAGE_ID"},
+            key_name="FAKE_KEY",
+            metadata={
+                "mkey": "mvalue",
+                "cluster_id": "CLUSTER_ID",
+                "cluster_node_id": "NODE_ID",
+                "cluster_node_index": 123
+            },
+            addresses={
+                "NET1": [{
+                    "OS-EXT-IPS:type": "fixed",
+                }],
+                "NET2": [{
+                    "OS-EXT-IPS:type": "fixed",
+                }],
+            },
+            security_groups=[{'name': 'GROUP1'}, {'name': 'GROUP2'}]
+        )
+        x_server.name = "FAKE_NAME"
+        cc = mock.Mock()
+        cc.server_get.return_value = x_server
+        profile._computeclient = cc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+        overrides = {
+            'networks': [{"network": "NET3"}]
+        }
+
+        res = profile.do_adopt(node_obj, overrides=overrides)
+
+        self.assertEqual(False, res['auto_disk_config'])
+        self.assertEqual('AZ01', res['availability_zone'])
+        self.assertEqual({'foo': 'bar'}, res['block_device_mapping_v2'])
+        self.assertFalse(res['config_drive'])
+        self.assertEqual('FLAVOR_ID', res['flavor'])
+        self.assertEqual('IMAGE_ID', res['image'])
+        self.assertEqual('FAKE_KEY', res['key_name'])
+        self.assertEqual({'mkey': 'mvalue'}, res['metadata'])
+        self.assertIn({'network': 'NET3'}, res['networks'])
+        self.assertNotIn({'network': 'NET1'}, res['networks'])
+        self.assertNotIn({'network': 'NET2'}, res['networks'])
+        self.assertIn('GROUP1', res['security_groups'])
+        self.assertIn('GROUP2', res['security_groups'])
+        cc.server_get.assert_called_once_with('FAKE_ID')
 
     def test_do_join_server_not_created(self):
         # Test path where server not specified
@@ -923,12 +1080,27 @@ class TestNovaServerBasic(base.SenlinTestCase):
         cc.server_get.assert_called_with('FAKE_ID')
         self.assertTrue(res)
 
+    @mock.patch.object(server.ServerProfile, 'do_delete')
+    @mock.patch.object(server.ServerProfile, 'do_create')
+    def test_do_recover_operation_is_none(self, mock_create, mock_delete):
+        profile = server.ServerProfile('t', self.spec)
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        mock_delete.return_value = None
+        mock_create.return_value = True
+
+        res = profile.do_recover(node_obj, operation=None)
+
+        self.assertTrue(res)
+        mock_delete.assert_called_once_with(node_obj, force=False)
+        mock_create.assert_called_once_with(node_obj)
+
     @mock.patch.object(server.ServerProfile, 'handle_rebuild')
     def test_do_recover_rebuild(self, mock_rebuild):
         profile = server.ServerProfile('t', self.spec)
         node_obj = mock.Mock(physical_id='FAKE_ID')
 
-        res = profile.do_recover(node_obj, operation='REBUILD')
+        res = profile.do_recover(node_obj, operation=[{'name': 'REBUILD'}])
 
         self.assertEqual(mock_rebuild.return_value, res)
         mock_rebuild.assert_called_once_with(node_obj)
@@ -938,21 +1110,30 @@ class TestNovaServerBasic(base.SenlinTestCase):
         profile = server.ServerProfile('t', self.spec)
         node_obj = mock.Mock(physical_id='FAKE_ID')
 
-        res = profile.do_recover(node_obj, operation=['REBUILD'])
+        res = profile.do_recover(node_obj, operation=[{'name': 'REBUILD'}])
 
         self.assertEqual(mock_rebuild.return_value, res)
         mock_rebuild.assert_called_once_with(node_obj)
 
+    @mock.patch.object(profiles_base.Profile, 'do_recover')
+    def test_do_recover_bad_operation(self, mock_base_recover):
+        profile = server.ServerProfile('t', self.spec)
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_recover(node_obj, operation=[{'name': 'BLAHBLAH'}])
+
+        self.assertFalse(res)
+
     @mock.patch.object(profiles_base.Profile, 'do_recover')
     def test_do_recover_fallback(self, mock_base_recover):
         profile = server.ServerProfile('t', self.spec)
         node_obj = mock.Mock(physical_id='FAKE_ID')
 
-        res = profile.do_recover(node_obj, operation='blahblah')
+        res = profile.do_recover(node_obj, operation=[{'name': 'RECREATE'}])
 
         self.assertEqual(mock_base_recover.return_value, res)
-        mock_base_recover.assert_called_once_with(node_obj,
-                                                  operation='blahblah')
+        mock_base_recover.assert_called_once_with(
+            node_obj, operation=[{'name': 'RECREATE'}])
 
     def test_handle_reboot(self):
         obj = mock.Mock(physical_id='FAKE_ID')
